 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1997-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*     faxroute.cpy                                                  */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*     This file contains the prototypes, etc for the                */
 /*     FAX routing extension API.                                    */
 /*                                                                   */
 /* --                                                                */


 %dcl #FAXROUTE_ char ext;
 %if #FAXROUTE_ = '' %then %do;
 %#FAXROUTE_ = 'Y';

 %dcl #INC_COMMCTRL char ext;
 %if #INC_COMMCTRL = '' %then %do;
 %include commctrl;
 %end;


 /* Microsoft routing guids */

 dcl MS_FAXROUTE_PRINTING_GUID TCHAR
     value ('{aec1b37c-9af2-11d0-abf7-00c04fd91a4e}');
 dcl MS_FAXROUTE_INBOX_GUID    TCHAR
     value ('{9d3d0c32-9af2-11d0-abf7-00c04fd91a4e}');
 dcl MS_FAXROUTE_FOLDER_GUID   TCHAR
     value ('{92041a90-9af2-11d0-abf7-00c04fd91a4e}');
 dcl MS_FAXROUTE_EMAIL_GUID    TCHAR
     value ('{6bbf7bfe-9af2-11d0-abf7-00c04fd91a4e}');


 /* callback routines */

 define alias PFAXROUTEADDFILE
        entry (type DWORD,
               type LPCWSTR,
               type GUID byaddr)
        returns (type LONG byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEDELETEFILE
        entry (type DWORD,
               type LPCWSTR)
        returns (type LONG byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEGETFILE
        entry (type DWORD,
               type DWORD,
               type LPWSTR,
               type DWORD byaddr)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEENUMFILE
        entry (type DWORD,
               type GUID byaddr,
               type GUID byaddr,
               type LPCWSTR,
               type PVOID)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEENUMFILES
        entry (type DWORD,
               type GUID byaddr,
               type PFAXROUTEENUMFILE,
               type PVOID)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEMODIFYROUTINGDATA
        entry (type DWORD,
               type LPCWSTR,
               type LPBYTE,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited WINAPI;


 define structure
   1 FAX_ROUTE_CALLBACKROUTINES,
     2 SizeOfStruct              type DWORD,
       /* size of the struct set by the fax service */
     2 FaxRouteAddFile           type PFAXROUTEADDFILE,
     2 FaxRouteDeleteFile        type PFAXROUTEDELETEFILE,
     2 FaxRouteGetFile           type PFAXROUTEGETFILE,
     2 FaxRouteEnumFiles         type PFAXROUTEENUMFILES,
     2 FaxRouteModifyRoutingData type PFAXROUTEMODIFYROUTINGDATA;
 define alias PFAX_ROUTE_CALLBACKROUTINES
        handle FAX_ROUTE_CALLBACKROUTINES;


 /* routing data structure */

 define structure
   1 FAX_ROUTE,
     2 SizeOfStruct    type DWORD,
       /* size of the struct set by the fax service */
     2 JobId           type DWORD,
       /* Fax job identifier */
     2 ElapsedTime     type DWORDLONG, /* Elapsed time for whole fax */
       /* job in Coordinated Universal Time (UTC) */
     2 ReceiveTime     type DWORDLONG, /* Starting time for fax job */
       /* in Coordinated Universal Time (UTC) */
     2 PageCount       type DWORD,/* Number of pages received */
     2 Csid            type LPCWSTR,/* Called station identifier */
     2 Tsid            type LPCWSTR,/* Transmitting station identifier*/
     2 CallerId        type LPCWSTR,/* Caller id */
     2 RoutingInfo     type LPCWSTR,
       /* Routing info such as DID, T.30-subaddress, etc. */
     2 ReceiverName    type LPCWSTR,  /* Receiver's name */
     2 ReceiverNumber  type LPCWSTR,/* Receiver's fax number */
     2 DeviceName      type LPCWSTR,
       /* Device name for the line that received the fax */
     2 DeviceId        type DWORD,
      /* Permanent line identifier for the receiving device */
     2 RoutingInfoData type LPBYTE,
       /* Routing infor data to override configured info */
     2 RoutingInfoDataSize type DWORD;/* Size of routing info data */

 define alias PFAX_ROUTE handle FAX_ROUTE;

 define ordinal FAXROUTE_ENABLE
   ( QUERY_STATUS value(-1),
     STATUS_DISABLE value(0),
     STATUS_ENABLE value(1)
   ) prec(31);


 /* prototypes */

 dcl FaxRouteInitialize
     entry (type HANDLE,
            type FAX_ROUTE_CALLBACKROUTINES byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('FaxRouteInitialize');

 dcl FaxRouteDeviceEnable
     entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
            type DWORD,
            type LONG)
     returns (type BOOL byvalue optional)
     WINAPI external('FaxRouteDeviceEnable');

 dcl FaxRouteDeviceChangeNotification
     entry (type DWORD,
            type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI external('FaxRouteDeviceChangeNotification');

 dcl FaxRouteGetRoutingInfo
     entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
            type DWORD,
            type LPBYTE,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('FaxRouteGetRoutingInfo');

 dcl FaxRouteSetRoutingInfo
     entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
            type DWORD,
            type LPBYTE, /* (*) type BYTE byaddr nonasgn */
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('FaxRouteSetRoutingInfo');


 define alias
        PFAXROUTEINITIALIZE
        entry  (type HANDLE,
                type FAX_ROUTE_CALLBACKROUTINES byaddr)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEMETHOD
        entry (type FAX_ROUTE byaddr,
               type PVOID byaddr,
               type DWORD byaddr)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias  PFAXROUTEDEVICEENABLE
        entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
               type DWORD,
               type LONG)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEDEVICECHANGENOTIFICATION
        entry (type DWORD,
               type BOOL)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTEGETROUTINGINFO
        entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
               type DWORD,
               type LPBYTE,
               type DWORD byaddr)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PFAXROUTESETROUTINGINFO
        entry (type LPCWSTR, /* WCHAR(*) varz byaddr nonasgn */
               type DWORD,
               type LPBYTE, /* (*) type BYTE byaddr nonasgn */
               type DWORD)
        returns (type BOOL byvalue optional)
        limited WINAPI;

 %end;
