 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1997  Microsoft Corporation                         */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   qos.cpy - QoS definitions for NDIS components.                  */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   This module defines the Quality of Service structures and types */
 /*   used by Winsock applications.                                   */
 /*                                                                   */
 /* Revision History:                                                 */
 /*                                                                   */
 /* --                                                                */

 %dcl #_QOS_H_ char ext;
 %if #_QOS_H_ = '' %then %do;
 %#_QOS_H_ = 'Y';


 /* Definitions for valued-based Service Type for each direction of   */
 /* data flow.                                                        */
 define alias SERVICETYPE type ULONG;

 dcl SERVICETYPE_NOTRAFFIC         type SERVICETYPE value('00000000'xu);
     /* No data in this direction */
 dcl SERVICETYPE_BESTEFFORT        type SERVICETYPE value('00000001'xu);
     /* Best Effort */
 dcl SERVICETYPE_CONTROLLEDLOAD    type SERVICETYPE value('00000002'xu);
     /* Controlled Load */
 dcl SERVICETYPE_GUARANTEED        type SERVICETYPE value('00000003'xu);
     /* Guaranteed */
 dcl SERVICETYPE_NETWORK_UNAVAILABLE type SERVICETYPE
                                                    value('00000004'xu);
     /* Used to notify change to user */
 dcl SERVICETYPE_GENERAL_INFORMATION type SERVICETYPE
                                                    value('00000005'xu);
     /* corresponds to "General Parameters" defined by IntServ */
 dcl SERVICETYPE_NOCHANGE          type SERVICETYPE value('00000006'xu);
     /* used to indicate that the flow spec contains no change */
     /* from any previous one                                  */

 dcl SERVICETYPE_NONCONFORMING     type SERVICETYPE value('00000009'xu);
     /* Non-Conforming Traffic */
 dcl SERVICETYPE_NETWORK_CONTROL   type SERVICETYPE value('0000000A'xu);
     /* Network Control traffic */
 dcl SERVICETYPE_QUALITATIVE       type SERVICETYPE value('0000000D'xu);
     /* Qualitative applications */


 /* Definitions for bitmap-based Service Type for each direction of */
 /* data flow.                                                      */

 /*********  The usage of these is currently not supported.  **********/
 dcl SERVICE_BESTEFFORT            type ULONG value('80020000'xu);
 dcl SERVICE_CONTROLLEDLOAD        type ULONG value('80040000'xu);
 dcl SERVICE_GUARANTEED            type ULONG value('80080000'xu);
 dcl SERVICE_QUALITATIVE           type ULONG value('80200000'xu);
 /*********  **********************************************  **********/



 /* Flags to control the usage of RSVP on this flow. */

 /* to turn off traffic control, 'OR' ( | ) this flag with the */
 /* ServiceType field in the FLOWSPEC                          */

 dcl SERVICE_NO_TRAFFIC_CONTROL    type ULONG value('81000000'xu);

 /* this flag can be used to prevent any rsvp signaling messages from */
 /* being sent. Local traffic control will be invoked, but no RSVP    */
 /* Path messages will be sent.This flag can also be used in          */
 /* conjunction with a receiving flowspec to suppress the automatic   */
 /* generation of a Reserve message. The application would receive    */
 /* notification that a Path message had arrived and would then need  */
 /* to alter the QOS by issuing WSAIoctl( SIO_SET_QOS ), to unset     */
 /* this flag and thereby causing Reserve messages to go out.         */

 dcl SERVICE_NO_QOS_SIGNALING      type ULONG value('40000000'xu);


 /* Flow Specifications for each direction of data flow. */
 define structure
   1 FLOWSPEC,
     2 TokenRate          type ULONG,       /* In Bytes/sec */
     2 TokenBucketSize    type ULONG,       /* In Bytes */
     2 PeakBandwidth      type ULONG,       /* In Bytes/sec */
     2 Latency            type ULONG,       /* In microseconds */
     2 DelayVariation     type ULONG,       /* In microseconds */
     2 ServiceType        type SERVICETYPE,
     2 MaxSduSize         type ULONG,       /* In Bytes */
     2 MinimumPolicedSize type ULONG;       /* In Bytes */
 define alias  PFLOWSPEC handle FLOWSPEC;
 define alias LPFLOWSPEC handle FLOWSPEC;

 /* This value can be used in the FLOWSPEC structure to instruct the  */
 /* Rsvp Service provider to derive the appropriate default value for */
 /* the parameter. Note that not all values in the FLOWSPEC structure */
 /* can be defaults. In the ReceivingFlowspec, all parameters can be  */
 /* defaulted except the ServiceType. In the SendingFlowspec, the     */
 /* MaxSduSize and MinimumPolicedSize can be defaulted. Other         */
 /* defaults may be possible. Refer to the appropriate documentation. */
 dcl QOS_NOT_SPECIFIED      type ULONG value('FFFFFFFF'xu);



 /* Define a value that can be used for the PeakBandwidth, which will */
 /* map into positive infinity when the FLOWSPEC is converted into    */
 /* IntServ floating point format. We can't use (-1) because that     */
 /* value was previously defined to mean "select the default".        */
 dcl POSITIVE_INFINITY_RATE type ULONG value('FFFFFFFE'xu);



 /* The provider specific structure can have a number of objects in   */
 /* it. Each next structure in the ProviderSpecific will be the       */
 /* QOS_OBJECT_HDR struct that prefaces the actual data with a type   */
 /* and length for that object.  This QOS_OBJECT structure can repeat */
 /* several times if there are several objects. This list of objects  */
 /* terminates either when the buffer length has been reached (WSABUF)*/
 /* or an object of type QOS_END_OF_LIST is encountered.              */
 define structure
   1 QOS_OBJECT_HDR,
     2 ObjectType   type ULONG,
     2 ObjectLength type ULONG;
       /* the length of object buffer INCLUDING this header */
 define alias LPQOS_OBJECT_HDR handle QOS_OBJECT_HDR;


 /* General QOS objects start at this offset from the base and have a */
 /* range of 1000                                                     */
 dcl QOS_GENERAL_ID_BASE fixed bin(31) value(2000);

 dcl ( QOS_OBJECT_END_OF_LIST    value('00000001'xu+QOS_GENERAL_ID_BASE)
       /* QOS_End_of_list structure passed */
     , QOS_OBJECT_SD_MODE        value('00000002'xu+QOS_GENERAL_ID_BASE)
       /* QOS_ShapeDiscard structure passed */
     , QOS_OBJECT_SHAPING_RATE   value('00000003'xu+QOS_GENERAL_ID_BASE)
       /* QOS_ShapingRate structure */
     , QOS_OBJECT_DESTADDR       value('00000004'xu+QOS_GENERAL_ID_BASE)
       /* QOS_DestAddr structure (defined in qossp.cpy) */
     ) type ULONG;



 /* This structure is used to define the behaviour that the traffic   */
 /* control packet shaper will apply to the flow.                     */
 /*                                                                   */
 /* TC_NONCONF_BORROW - the flow will receive resources remaining     */
 /*  after all higher priority flows have been serviced. If a         */
 /*  TokenRate is specified, packets may be non-conforming and        */
 /*  will be demoted to less than best-effort priority.               */
 /*                                                                   */
 /* TC_NONCONF_SHAPE - TokenRate must be specified. Non-conforming    */
 /*  packets will be retianed in the packet shaper until they become  */
 /*  conforming.                                                      */
 /*                                                                   */
 /* TC_NONCONF_DISCARD - TokenRate must be specified. Non-conforming  */
 /*  packets will be discarded.                                       */

 define structure
   1 QOS_SD_MODE,
     2 ObjectHdr        type QOS_OBJECT_HDR,
     2 ShapeDiscardMode type ULONG;
 define alias LPQOS_SD_MODE handle QOS_SD_MODE;

 dcl TC_NONCONF_BORROW      type ULONG value(0);
 dcl TC_NONCONF_SHAPE       type ULONG value(1);
 dcl TC_NONCONF_DISCARD     type ULONG value(2);
 dcl TC_NONCONF_BORROW_PLUS type ULONG value(3);


 /* This structure allows an app to specify a prorated "average token */
 /* rate" using by the traffic shaper under SHAPE modehaper queue.    */
 /* It is expressed in bytes per sec.                                 */

 /* ShapingRate (bytes per sec.)                                      */

 define structure
   1 QOS_SHAPING_RATE,
     2 ObjectHdr   type QOS_OBJECT_HDR,
     2 ShapingRate type ULONG;
 define alias LPQOS_SHAPING_RATE handle QOS_SHAPING_RATE;

 %end; /* __QOS_H_ */
