 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1992-1998 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   rpcndr.cpy                                                      */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Definitions for stub data structures and prototypes of helper   */
 /*   functions.                                                      */
 /*                                                                   */
 /* --                                                                */

 /* This version of the rpcndr.h file corresponds to MIDL version     */
 /* 5.0.+ used with NT5 beta1+ env from build #1700 on.               */


 %dcl #_RPCNDR_H_VERSION__ fixed ext;
 %if #_RPCNDR_H_VERSION__ = 0 %then %do;
   %#_RPCNDR_H_VERSION__ = 475;
 %end; /* __RPCNDR_H_VERSION__ */


 %dcl #_RPCNDR_H__ char ext;
 %if #_RPCNDR_H__ = '' %then %do;
 %#_RPCNDR_H__ = 'Y';

 %dcl #_REQUIRED_RPCNDR_H_VERSION__ fixed ext;
 %if #_REQUIRED_RPCNDR_H_VERSION__ ^= 0 %then %do;
   %if  #_RPCNDR_H_VERSION__ < #_REQUIRED_RPCNDR_H_VERSION__ %then %do;
     %note('Incorrect rpcndr.cpy version.' !!
           'Use the header that matches with the MIDL compiler.',8);
   %end;
 %end;

 %dcl #BASETSD_H_ char ext;
 %if #BASETSD_H_ = '' %then %do;
 %include basetsd;
 %end;
 %dcl #_RPCNSIP_H__ char ext;
 %if #_RPCNSIP_H__ = '' %then %do;
 %include rpcnsip;
 %end;


 /*********************************************************************/
 /*                                                                   */
 /*  Network Computing Architecture (NCA) definition:                 */
 /*                                                                   */
 /*  Network Data Representation: (NDR) Label format:                 */
 /*  An unsigned long (32 bits) with the following layout:            */
 /*                                                                   */
 /*  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1                      */
 /*  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0  */
 /* +---------------+---------------+---------------+-------+-------+ */
 /* |   Reserved    |   Reserved    |Floating point | Int   | Char  | */
 /* |               |               |Representation | Rep.  | Rep.  | */
 /* +---------------+---------------+---------------+-------+-------+ */
 /*                                                                   */
 /*  Where                                                            */
 /*                                                                   */
 /*      Reserved:                                                    */
 /*                                                                   */
 /*          Must be zero (0) for NCA 1.5 and NCA 2.0.                */
 /*                                                                   */
 /*      Floating point Representation is:                            */
 /*                                                                   */
 /*          0 - IEEE                                                 */
 /*          1 - VAX                                                  */
 /*          2 - Cray                                                 */
 /*          3 - IBM                                                  */
 /*                                                                   */
 /*      Int Rep. is Integer Representation:                          */
 /*                                                                   */
 /*          0 - Big Endian                                           */
 /*          1 - Little Endian                                        */
 /*                                                                   */
 /*      Char Rep. is Character Representation:                       */
 /*                                                                   */
 /*          0 - ASCII                                                */
 /*          1 - EBCDIC                                               */
 /*                                                                   */
 /*  The Microsoft Local Data Representation (for all platforms which */
 /*  are of interest) currently is defined below:                     */
 /*                                                                   */
 /*********************************************************************/

 dcl NDR_CHAR_REP_MASK  fixed bin(31) value('0000000F'xn);
 dcl NDR_INT_REP_MASK   fixed bin(31) value('000000F0'xn);
 dcl NDR_FLOAT_REP_MASK fixed bin(31) value('0000FF00'xn);

 dcl NDR_LITTLE_ENDIAN  fixed bin(31) value('00000010'xn);
 dcl NDR_BIG_ENDIAN     fixed bin(31) value('00000000'xn);

 dcl NDR_IEEE_FLOAT     fixed bin(31) value('00000000'xn);
 dcl NDR_VAX_FLOAT      fixed bin(31) value('00000100'xn);
 dcl NDR_IBM_FLOAT      fixed bin(31) value('00000300'xn);

 dcl NDR_ASCII_CHAR     fixed bin(31) value('00000000'xn);
 dcl NDR_EBCDIC_CHAR    fixed bin(31) value('00000001'xn);

 dcl NDR_LOCAL_DATA_REPRESENTATION fixed bin(31) value('00000010'xn);
 dcl NDR_LOCAL_ENDIAN   fixed bin (31) value (NDR_LITTLE_ENDIAN);


 /*********************************************************************/
 /*  Macros for targeted platforms                                    */
 /*********************************************************************/

 %dcl WINVER       fixed ext;
 %dcl #WIN32_WINNT fixed ext;
 %dcl #WIN32_DCOM  char ext;

 %dcl TARGET_IS_NT50_OR_LATER           fixed ext;
 %dcl TARGET_IS_NT40_OR_LATER           fixed ext;
 %dcl TARGET_IS_NT351_OR_WIN95_OR_LATER fixed ext;

 %if #WIN32_WINNT >= 1280 %then %do;
   %TARGET_IS_NT50_OR_LATER = 1;
 %end; %else %do;
   %TARGET_IS_NT50_OR_LATER = 0;
 %end;

 %if #WIN32_DCOM ^= '' | #WIN32_WINNT >= 1024 %then %do;
   %TARGET_IS_NT40_OR_LATER = 1;
 %end; %else %do;
   %TARGET_IS_NT40_OR_LATER = 0;
 %end;

 %if WINVER >= 1024 %then %do;
   %TARGET_IS_NT351_OR_WIN95_OR_LATER = 1;
 %end; %else %do;
   %TARGET_IS_NT351_OR_WIN95_OR_LATER = 0;
 %end;

 /*********************************************************************/
 /*  Other MIDL base types / predefined types:                        */
 /*********************************************************************/

 define alias small   fixed bin(7);
 /* define alias byte    fixed bin (7) signed   */
 /* define alias boolean fixed bin (8) unsigned */

 %dcl #HYPER_DEFINED char ext;
 %if #HYPER_DEFINED = '' %then %do;
 %dcl #HYPER_DEFINED char ext;
 %#HYPER_DEFINED = 'Y';

 define alias hyper fixed bin (63);
 %end; /* _HYPER_DEFINED */

 %dcl #WCHAR_T_DEFINED char ext;
 %if #WCHAR_T_DEFINED = '' %then %do;
 %dcl WCHAR_T char;
 %WCHAR_T = 'WIDECHAR';
 %#WCHAR_T_DEFINED = 'Y';
 %end;

 %dcl #SIZE_T_DEFINED char ext;
 %if #SIZE_T_DEFINED = '' %then %do;
 define alias size_t fixed bin(31);
 %#SIZE_T_DEFINED = 'Y';
 %end;


 %dcl #_RPC_WIN32__ char ext;
 %if #_RPC_WIN32__ ^= '' %then %do;
 %dcl #_RPC_CALLEE char ext;
 %#_RPC_CALLEE = 'options (byvalue linkage(stdcall) nodescriptor)';
 %end;

 dcl MIDL_user_allocate
     entry (type size_t )
     returns (pointer byvalue optional)
     #_RPC_USER external('MIDL_user_allocate');
 dcl MIDL_user_free
     entry (pointer)
     #_RPC_USER external('MIDL_user_free');

 %dcl RPC_VAR_ENTRY char ext;
 %RPC_VAR_ENTRY = 'options (byvalue linkage(cdecl) nodescriptor)';

 /* winnt only */
 %dcl #_MIDL_DECLSPEC_DLLIMPORT char ext;
 %#_MIDL_DECLSPEC_DLLIMPORT = '';
 %dcl #_MIDL_DECLSPEC_DLLEXPORT  char ext;
 %#_MIDL_DECLSPEC_DLLEXPORT = '';


 /*********************************************************************/
 /* Context handle management related definitions:                    */
 /*                                                                   */
 /* Client and Server Contexts.                                       */
 /*                                                                   */
 /*********************************************************************/

 define alias NDR_CCONTEXT pointer;

 define structure
   1 #NDR_SCONTEXT#, /* anonymous type */
     2 pad (2)     pointer,
     2 userContext pointer;
 define alias NDR_SCONTEXT handle #NDR_SCONTEXT#;

 /* Use both as function and pseudo-variable */
 %NDRSContextValue: proc (hContext) returns (char);
   dcl hContext char; /* type NDR_SCONTEXT */
   return (hContext || ' => userContext');
 %end NDRSContextValue;

 dcl cbNDRContext fixed bin(31) value(20); /* size of context on WIRE */

 define alias NDR_RUNDOWN
        entry (pointer)
        limited #_RPC_USER;

 define alias NDR_NOTIFY_ROUTINE
        entry ()
        limited #_RPC_USER;

 define alias NDR_NOTIFY2_ROUTINE
        entry (type boolean)
        limited #_RPC_USER;

 define structure
   1 SCONTEXT_QUEUE,
     2 NumberOfObjects fixed bin(31),
     2 ArrayOfObjects  pointer;
 define alias PSCONTEXT_QUEUE handle SCONTEXT_QUEUE;

 dcl NDRCContextBinding
     entry (type NDR_CCONTEXT)
     returns (type RPC_BINDING_HANDLE byvalue optional)
     RPCRTAPI RPC_ENTRY external('NDRCContextBinding');

 dcl NDRCContextMarshall
     entry (type NDR_CCONTEXT,
            pointer)
     RPCRTAPI RPC_ENTRY external('NDRCContextMarshall');

 dcl NDRCContextUnmarshall
     entry (type NDR_CCONTEXT byaddr,
            type RPC_BINDING_HANDLE,
            pointer,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NDRCContextUnmarshall');

 dcl NDRSContextMarshall
     entry (type NDR_SCONTEXT,
            pointer,
            type NDR_RUNDOWN)
     RPCRTAPI RPC_ENTRY external('NDRSContextMarshall');

 dcl NDRSContextUnmarshall
     entry (pointer,
            fixed bin(31))
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NDRSContextUnmarshall');

 dcl NDRSContextMarshallEx
     entry (type RPC_BINDING_HANDLE,
            type NDR_SCONTEXT,
            pointer,
            type NDR_RUNDOWN)
     RPCRTAPI RPC_ENTRY external('NDRSContextMarshallEx');

 dcl NDRSContextMarshall2
     entry (type RPC_BINDING_HANDLE,
            type NDR_SCONTEXT,
            pointer,
            type NDR_RUNDOWN,
            pointer,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NDRSContextMarshall2');

 dcl NDRSContextUnmarshallEx
     entry (type RPC_BINDING_HANDLE,
            pointer,
            fixed bin(31))
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NDRSContextUnmarshallEx');

 dcl NDRSContextUnmarshall2
     entry (type RPC_BINDING_HANDLE,
            pointer,
            fixed bin(31),
            pointer,
            fixed bin(31))
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NDRSContextUnmarshall2');

 dcl RpcSsDestroyClientContext
     entry (pointer byaddr)
     RPCRTAPI RPC_ENTRY external('RpcSsDestroyClientContext');


 /*********************************************************************/
 /*   NDR conversion related definitions.                             */
 /*********************************************************************/

 byte_from_ndr: proc (source, target)
                options (inline reorder) internal;
   dcl source  type RPC_MESSAGE byaddr,
       target  type byte byaddr;
   dcl bytebuf type byte based;
   target = source.Buffer -> bytebuf;
   source.buffer += 1;
 end byte_from_ndr;

 byte_array_from_ndr: proc (Source, LowerIndex, UpperIndex, Target)
                      options (inline reorder) internal;
   dcl Source     type RPC_MESSAGE byaddr,
       LowerIndex fixed bin (31) byvalue,
       UpperIndex fixed bin (31) byvalue,
       Target     (*) type byte byaddr;
   call NDRcopy (addr (Target) + LowerIndex, Source.Buffer,
                 UpperIndex - LowerIndex);
   Source.Buffer += UpperIndex - LowerIndex;
 end byte_array_from_ndr;

 boolean_from_ndr: proc (source, target)
                   options (inline reorder) internal;
   dcl source  type RPC_MESSAGE byaddr,
       target  type boolean byaddr;
   dcl booleanbuf type boolean based;
   target = source.Buffer -> booleanbuf;
   source.buffer += 1;
 end boolean_from_ndr;

 boolean_array_from_ndr: proc (Source, LowerIndex, UpperIndex, Target)
                         options (inline reorder) internal;
   dcl Source     type RPC_MESSAGE byaddr,
       LowerIndex fixed bin (31) byvalue,
       UpperIndex fixed bin (31) byvalue,
       Target     (*) type boolean byaddr;
   call NDRcopy (addr (Target) + LowerIndex, Source.Buffer,
                 UpperIndex - LowerIndex);
   Source.Buffer += UpperIndex - LowerIndex;
 end boolean_array_from_ndr;

 small_from_ndr: proc (source, target)
                 options(inline reorder) internal;
   dcl source  type RPC_MESSAGE byaddr,
       target  type small byaddr;
   dcl smallbuf type small based;
   target = source.Buffer -> smallbuf;
   source.buffer += 1;
 end small_from_ndr;

 small_from_ndr_temp: proc (source, target, format)
                      options(inline reorder) internal;
   dcl source  pointer byaddr, /* is RPC_MESSAGE.Buffer */
       target  type small byaddr,
       format  fixed bin (31) byvalue; /* ignored */
   dcl smallbuf type small based;
   target = source -> smallbuf;
   source += 1;
 end small_from_ndr_temp;


 small_array_from_ndr: proc (Source, LowerIndex, UpperIndex, Target)
                       options (inline reorder) internal;
   dcl Source     type RPC_MESSAGE byaddr,
       LowerIndex fixed bin (31) byvalue,
       UpperIndex fixed bin (31) byvalue,
       Target     (*) type small byaddr;
   call NDRcopy (addr (Target) + LowerIndex, Source.Buffer,
                 UpperIndex - LowerIndex);
   Source.Buffer += UpperIndex - LowerIndex;
 end small_array_from_ndr;


 /*********************************************************************/
 /* Mapping of PL/I-runtime functions.                                */
 /*********************************************************************/

 MIDL_ascii_strlen: proc (string)
                    returns (fixed bin (31) byvalue)
                    options (inline reorder) internal;
   dcl string char (*) varz byaddr nonasgn;
   dcl length builtin;
   return (length (string));
 end MIDL_ascii_strlen;

 MIDL_ascii_strcpy: proc (target, source)
                    options (inline reorder) internal;
   dcl target char (*) varz byaddr,
       source char (*) varz byaddr nonasgn;
   target = source;
 end MIDL_ascii_strcpy;

 MIDL_memset: proc (s, c, n)
               options (inline reorder) internal;
   dcl s pointer byvalue,     /* destination */
       c char byvalue,        /* character */
       n type size_t byvalue; /* count */
   dcl plifill builtin;
   call plifill (s, c, n);
 end MIDL_memset;


 /*********************************************************************/
 /* Ndr Library helper function prototypes for MIDL 1.0 ndr functions.*/
 /*********************************************************************/

 dcl NDRcopy
     entry (pointer,
            pointer,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NDRcopy');

 dcl MIDL_wchar_strlen
     entry (WCHAR_T(*) varz byaddr)
     returns (type size_t byvalue optional)
     RPCRTAPI RPC_ENTRY external('MIDL_wchar_strlen');

 dcl MIDL_wchar_strcpy
     entry (pointer,
            WCHAR_T(*) varz byaddr)
     RPCRTAPI RPC_ENTRY external('MIDL_wchar_strcpy');

 dcl char_from_ndr
     entry (type RPC_MESSAGE byaddr,
            char (1) byaddr)
     RPCRTAPI RPC_ENTRY external('char_from_ndr');

 dcl char_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            char (*) byaddr)
     RPCRTAPI RPC_ENTRY external('char_array_from_ndr');

 dcl short_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(16) unsigned byaddr)
     RPCRTAPI RPC_ENTRY external('short_from_ndr');

 dcl short_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            (*) fixed bin(16) unsigned byaddr)
     RPCRTAPI RPC_ENTRY external('short_array_from_ndr');

 dcl short_from_ndr_temp
     entry (pointer byaddr,
            fixed bin(16) unsigned byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('short_from_ndr_temp');

 dcl long_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     RPCRTAPI RPC_ENTRY external('long_from_ndr');

 dcl long_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            (*) fixed bin(31) byaddr)
     RPCRTAPI RPC_ENTRY external('long_array_from_ndr');

 dcl long_from_ndr_temp
     entry (pointer byaddr,
            fixed bin(31) byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('long_from_ndr_temp');

 dcl enum_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     RPCRTAPI RPC_ENTRY external('enum_from_ndr');

 dcl float_from_ndr
     entry (type RPC_MESSAGE byaddr,
            float bin(21) byaddr)
     RPCRTAPI RPC_ENTRY external('float_from_ndr');

 dcl float_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            (*) float bin(21) byaddr)
     RPCRTAPI RPC_ENTRY external('float_array_from_ndr');

 dcl double_from_ndr
     entry (type RPC_MESSAGE byaddr,
            float bin(53) byaddr)
     RPCRTAPI RPC_ENTRY external('double_from_ndr');

 dcl double_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            (*) float bin(53) byaddr)
     RPCRTAPI RPC_ENTRY external('double_array_from_ndr');

 dcl hyper_from_ndr
     entry (type RPC_MESSAGE byaddr,
            type hyper byaddr)
     RPCRTAPI RPC_ENTRY external('hyper_from_ndr');

 dcl hyper_array_from_ndr
     entry (type RPC_MESSAGE byaddr,
            fixed bin(31),
            fixed bin(31),
            type hyper byaddr)
     RPCRTAPI RPC_ENTRY external('hyper_array_from_ndr');

 dcl hyper_from_ndr_temp
     entry (pointer byaddr,
            type hyper byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('hyper_from_ndr_temp');

 dcl data_from_ndr
     entry (type RPC_MESSAGE byaddr,
            pointer,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('data_from_ndr');

 dcl data_into_ndr
     entry (pointer,
            type RPC_MESSAGE byaddr,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('data_into_ndr');

 dcl tree_into_ndr
     entry (pointer,
            type RPC_MESSAGE byaddr,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('tree_into_ndr');

 dcl data_size_ndr
     entry (pointer,
            type RPC_MESSAGE byaddr,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('data_size_ndr');

 dcl tree_size_ndr
     entry (pointer,
            type RPC_MESSAGE byaddr,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('tree_size_ndr');

 dcl tree_peek_ndr
     entry (type RPC_MESSAGE byaddr,
            pointer,
            char (*) varz byaddr,
            char)
     RPCRTAPI RPC_ENTRY external('tree_peek_ndr');

 dcl midl_allocate
     entry (type size_t)
     returns (pointer byvalue optional)
     RPCRTAPI RPC_ENTRY external('midl_allocate');

 /*********************************************************************/
 /*   MIDL 2.0 ndr definitions.                                       */
 /*********************************************************************/

 define alias error_status_t fixed bin(31);

 /*********************************************************************/
 /* PL/I NOTE:                                                        */
 /*                                                                   */
 /* The marshalling macros:                                           */
 /*   _midl_ma1,                                                      */
 /*   _midl_ma2,                                                      */
 /*   _midl_ma3,                                                      */
 /*   _midl_ma4                                                       */
 /*   _midl_marsh_lhs                                                 */
 /* and the unmarshalling macros:                                     */
 /*   _midl_unma1                                                     */
 /*   _midl_unma2                                                     */
 /*   _midl_unma3                                                     */
 /*   _midl_unma4                                                     */
 /* cannot be converted into PL/I, as there is no CAST (: :) type BIF */
 /* and BIND (: :) is only for structure types (it returns a handle). */
 /* Such a CAST (: :) would have to accept full type dcls             */
 /*                                                                   */
 /* Nevertheless, translations of:                                    */
 /*   NdrMarshConfStringHdr                                           */
 /*   NdrUnMarshConfStringHdr                                         */
 /* are provided.                                                     */
 /*********************************************************************/

 /* Some alignment specific macros. */

 #midl_fa2: proc (p) returns (pointer byvalue)
            options(inline reorder) internal;
   dcl p pointer byvalue;
   dcl (binvalue, iand, inot, ptrvalue) builtin;
   return (ptrvalue (iand (binvalue (p) + 1, inot('1'xn))));
 end #midl_fa2;

 #midl_fa4: proc (p) returns (pointer byvalue)
            options(inline reorder) internal;
   dcl p pointer byvalue;
   dcl (binvalue, iand, inot, ptrvalue) builtin;
   return (ptrvalue (iand (binvalue (p) + 3, inot('3'xn))));
 end #midl_fa4;

 #midl_fa8: proc (p) returns (pointer byvalue)
            options(inline reorder) internal;
   dcl p pointer byvalue;
   dcl (binvalue, iand, inot, ptrvalue) builtin;
   return (ptrvalue (iand (binvalue (p) + 7, inot('7'xn))));
 end #midl_fa8;

 #midl_addp: proc (p, n) returns (pointer byvalue optional)
             options(inline reorder) internal;
   dcl p pointer byaddr,
       n type ULONG_PTR byvalue;
   p += n;
   return (p);
 end;


 /* Marshalling macros */

 #midl_marsh_up: proc (mp, p)
                 options (inline reorder) internal;
   dcl mp pointer byaddr,
       p  pointer byvalue;
   dcl ulongbuf fixed bin (31) based;
   dcl stg builtin;

   mp -> ulongbuf = binvalue (p);
   mp += stg (mp -> ulongbuf);
 end #midl_marsh_up;

 #midl_advmp: proc (mp) returns (fixed bin(31) optional byvalue)
              options (inline reorder) internal;
   dcl mp pointer byaddr;
   dcl ulongbuf fixed bin (31) based,
       res fixed bin (31);
   dcl stg builtin;
   res = mp -> ulongbuf;
   mp += stg (mp -> ulongbuf);
   return (res);
 end #midl_advmp;

 #midl_unmarsh_up: proc (p) returns (fixed bin(31) optional byvalue)
                   options (inline reorder) internal;
   dcl p pointer byaddr;
   dcl ulongbuf fixed bin (31) based,
       res fixed bin (31);
   dcl stg builtin;
   res = p -> ulongbuf;
   p += stg (p -> ulongbuf);
   return (res);
 end #midl_unmarsh_up;


 /*****************************************************************/
 /* Ndr macros.                                                   */
 /*****************************************************************/

 NdrMarshConfStringHdr: proc (p, s, l)
                        options (inline reorder) internal;
   dcl p pointer byaddr,
       s fixed bin (31) byvalue,
       l fixed bin (31) byvalue;
   dcl ulongbuf fixed bin (31) based;
   dcl stg builtin;

   p -> ulongbuf = s;
   p += stg (p -> ulongbuf);
   p -> ulongbuf = 0;
   p += stg (p -> ulongbuf);
   p -> ulongbuf = l;
   p += stg (p -> ulongbuf);
 end NdrMarshConfStringHdr;

 NdrUnMarshConfStringHdr: proc (p, s, l)
                          options (inline reorder) internal;
   dcl p pointer byaddr,
       s fixed bin (31) byaddr,
       l fixed bin (31) byaddr;
   dcl ulongbuf fixed bin (31) based;
   dcl stg builtin;
   s = p -> ulongbuf;
   p += stg (p -> ulongbuf);
   p += stg (p -> ulongbuf);
   l = p -> ulongbuf;
   p += stg (p -> ulongbuf);
 end NdrUnMarshConfStringHdr;


 NdrMarshCCtxtHdl: proc (pc, p)
                   returns (pointer byvalue optional)
                   options (inline reorder) internal;
   dcl pc type NDR_CCONTEXT,
       p  pointer byvalue;
  call NDRCContextMarshall (pc, p );
  return (p + 20);
 end NdrMarshCCtxtHdl;

 NdrUnMarshCCtxtHdl: proc (pc, p, h, drep)
                     returns (pointer byvalue optional)
                     options (inline reorder) internal;
   dcl pc   type NDR_CCONTEXT byaddr,
       p    pointer byvalue,
       h    type RPC_BINDING_HANDLE byvalue,
       drep fixed bin(31) byvalue;
   call NDRCContextUnmarshall (pc, h, p, drep);
   return (p + 20);
 end NdrUnMarshCCtxtHdl;


 NdrUnMarshSCtxtHdl: proc (pc, p, drep)
                     options (inline reorder) internal;
   dcl pc   type NDR_CCONTEXT byaddr,
       p    pointer byvalue,
       drep fixed bin(31) byvalue;
   pc = NdrSContextUnMarshall (p, drep);
 end NdrUnMarshSCtxtHdl;

 NdrMarshSCtxtHdl: proc (pc, p, rd)
                   options (inline reorder) internal;
   dcl pc type NDR_SCONTEXT byvalue,
       p  pointer byvalue,
       rd type NDR_RUNDOWN byvalue;
   call NdrSContextMarshall (pc, p, rd);
 end NdrMarshSCtxtHdl;


 %NdrFieldOffset: proc (s, f) returns (char);
   dcl (s, f) char;
   return ('location (bind (: ' || s || ', sysnull() :) => ' ||
                      f || ')');
 %end NdrFieldOffset;
 %dcl NdrFieldOffset entry;

 %NdrFieldPad: proc (s, f, p, t) returns (char);
   dcl (s, f, p, t) char;
    return ('(' || NdrFieldOffset (s ,f) || ' - '
                || NdrFieldOffset (s, p) || ' - '
                || 'size (: ' || t || ' :) )');
 %end NdrFieldPad;
 %dcl NdrFieldPad entry;


 /* On the server side, the following exceptions are mapped to */
 /* the bad stub data exception if -error stub_data is used. */

 %dcl RPC_BAD_STUB_DATA_EXCEPTION_FILTER char ext;
 %RPC_BAD_STUB_DATA_EXCEPTION_FILTER =
    '((RpcExceptionCode() = STATUS_ACCESS_VIOLATION)      | ' ||
     '(RpcExceptionCode() = STATUS_DATATYPE_MISALIGNMENT) | ' ||
     '(RpcExceptionCode() = RPC_X_BAD_STUB_DATA)          | ' ||
     '(RpcExceptionCode() = RPC_S_INVALID_BOUND))';



 /*****************************************************************/
 /* Some stub helper functions.                                   */
 /*****************************************************************/

 /*****************************************************************/
 /* Stub helper structures.                                       */
 /*****************************************************************/

 define alias RPC_BUFPTR pointer;
 define alias RPC_LENGTH fixed bin(31);

 /* Expression evaluation callback routine prototype. */
 define alias EXPR_EVAL
        entry (handle MIDL_STUB_MESSAGE)
        limited #_RPC_USER;

 define alias PFORMAT_STRING pointer;

 /* Multidimensional conformant/varying array struct. */
 define structure
   1 ARRAY_INFO,
     2 Dimension             fixed bin(31),

     /* These fields MUST be (unsigned long *) */
     2 BufferConformanceMark pointer,
     2 BufferVarianceMark    pointer,

     /* Count arrays, used for top level arrays in -Os stubs */
     2 MaxCountArray         pointer,
     2 OffsetArray           pointer,
     2 ActualCountArray      pointer;
 define alias PARRAY_INFO handle ARRAY_INFO;

 /* Pipe related definitions. */

 define alias PNDR_PIPE_DESC        pointer;
 define alias PNDR_PIPE_MESSAGE     pointer;

 define alias PNDR_ASYNC_MESSAGE    pointer;
 define alias PNDR_CORRELATION_INFO pointer;

 define structure 1 IRpcChannelBuffer;

 /* MIDL Stub Message */

 define structure
   1 MIDL_STUB_MESSAGE,
     /* RPC message structure.                                        */
     2 RpcMsg                 type PRPC_MESSAGE,
     /* Pointer into RPC message buffer.                              */
     2 Buffer                 pointer,
     /* These are used internally by the Ndr routines to mark the     */
     /* beginning and end of an incoming RPC buffer.                  */
     2 BufferStart            pointer,
     2 BufferEnd              pointer,
     /* Used internally by the Ndr routines as a place holder in the  */
     /* buffer. On the marshalling side it's used to mark the         */
     /* location where conformance size should be marshalled.         */
     /* On the unmarshalling side it's used to mark the location in   */
     /* the buffer used during pointer unmarshalling to base pointer  */
     /* offsets off of.                                               */
     2 BufferMark             pointer,
     /* Set by the buffer sizing routines.                            */
     2 BufferLength           fixed bin(31),
     /* Set by the memory sizing routines.                            */
     2 MemorySize             fixed bin(31),
     /* Pointer to user memory.                                       */
     2 Memory                 pointer,
     /* Is the Ndr routine begin called from a client side stub.      */
     2 IsClient               fixed bin(31),
     /* Can the buffer be re-used for memory on unmarshalling.        */
     2 ReuseBuffer            fixed bin(31),
     /* Holds the current pointer to an allocate all nodes memory     */
     /* block.                                                        */
     2 AllocAllNodesMemory    pointer,
     /* Used for debugging asserts only, remove later.                */
     2 AllocAllNodesMemoryEnd pointer,
     /* Stuff needed while handling complex structures. Ignore        */
     /* imbeded pointers while computing buffer or memory sizes.      */
     2 IgnoreEmbeddedPointers fixed bin(31),
     /* This marks the location in the buffer where pointees of a     */
     /* complex struct reside.                                        */
     2 PointerBufferMark      pointer,
     /* Used to catch errors in SendReceive.                          */
     2 fBufferValid           fixed bin(8) unsigned,
     /* Obsolete unused field (formerly MaxContextHandleNumber).      */
     2 uFlags                 fixed bin(8) unsigned,
     /* Used internally by the Ndr routines. Holds the max counts     */
     /* for a conformant array.                                       */
     2 MaxCount               fixed bin(31),
     /* Used internally by the Ndr routines. Holds the offsets for    */
     /* a varying array.                                              */
     2 Offset                 fixed bin(31),
     /* Used internally by the Ndr routines. Holds the actual counts  */
     /* for a varying array.                                          */
     2 ActualCount            fixed bin(31),
     /* Allocation and Free routine to be used by the Ndr routines.   */
     2 pfnAllocate            entry (type size_t)
                              returns (pointer byvalue)
                              limited #_RPC_API,
     2 pfnFree                entry (pointer)
                              limited #_RPC_API,
     /* Top of parameter stack. Used for "single call" stubs during   */
     /* marshalling to hold the beginning of the parameter list on    */
     /* the stack. Needed to extract parameters which hold attribute  */
     /* values for top level arrays and pointers.                     */
     2 StackTop               pointer,
     /* Fields used for the transmit_as and represent_as objects.     */
     /* For represent_as the mapping is: presented=local,             */
     /* transmit=named.                                               */
     2 pPresentedType         pointer,
     2 pTransmitType          pointer,
     /* When we first construct a binding on the client side, stick   */
     /* it in the rpcmessage and later call RpcGetBuffer, the handle  */
     /* field in the rpcmessage is changed. That's fine except that   */
     /* we need to have that original handle for use in unmarshalling */
     /* context handles (the second argument in NDRCContextUnmarshall */
     /* to be exact). So stash the contructed handle here and extract */
     /* it when needed.                                               */
     2 SavedHandle            type handle_t,
     /* Pointer back to the stub descriptor. Use this to get all      */
     /* handle info.                                                  */
     2 StubDesc               handle MIDL_STUB_DESC,
     /* Full pointer stuff.                                           */
     2 FullPtrXlatTables      handle FULL_PTR_XLAT_TABLES,
     2 FullPtrRefId           fixed bin(31),
     /* flags (INTEL byte order for PL/I)                             */
     2 PointeeBufferLength    fixed bin(31),
     2 *                      bit(16), /* PL/I padding */
     2 fUnused                bit(10),
     2 fHasNewCorrDesc        bit(1),
     2 fHasExtensions         bit(1),
     2 fHasReturn             bit(1),
     2 fInOnlyParam           bit(1),
     2 fDontCallFreeInst      bit(1),
     2 fInDontFree            bit(1),
     2 dwDestContext          fixed bin(31),
     2 pvDestContext          pointer,
     2 SavedContextHandles    pointer,
     2 ParamNumber fixed      bin(31),
     2 pRpcChannelBuffer      handle IRpcChannelBuffer,
     2 pArrayInfo             type PARRAY_INFO,
     /* This is where the Beta2 stub message ends.                   */
     2 SizePtrCountArray      pointer,
     2 SizePtrOffsetArray     pointer,
     2 SizePtrLengthArray     pointer,
     /* Interpreter argument queue. Used on server side only.        */
     2 pArgQueue              pointer,
     2 dwStubPhase            fixed bin(31),
     /* Pipe descriptor, defined for the 4.0 release.                */
     2 pPipeDesc              type PNDR_PIPE_DESC,
     /* Async message pointer, correlation data - NT 5.0 features.   */
     2 pAsyncMsg type         PNDR_ASYNC_MESSAGE,
     2 pCorrInfo type         PNDR_CORRELATION_INFO,
     2 pCorrMemory            pointer,
     2 Reserved (1)           type ULONG_PTR;
     /* Fields up to this point present since the 3.50 release.       */

 define alias PMIDL_STUB_MESSAGE handle MIDL_STUB_MESSAGE;

 /* Generic handle bind/unbind routine pair. */

 define alias GENERIC_BINDING_ROUTINE
        entry (pointer)
        returns (pointer byvalue optional)
        limited #_RPC_API;

 define alias GENERIC_UNBIND_ROUTINE
        entry (pointer,
               pointer)
        limited #_RPC_API;

 define structure
   1 GENERIC_BINDING_ROUTINE_PAIR,
    2 pfnBind   type GENERIC_BINDING_ROUTINE,
    2 pfnUnbind type GENERIC_UNBIND_ROUTINE;
 define alias PGENERIC_BINDING_ROUTINE_PAIR
        handle GENERIC_BINDING_ROUTINE_PAIR;

 define structure
   1 GENERIC_BINDING_INFO,
    2 pObj      pointer,
    2 Size      fixed bin(31),
    2 pfnBind   type GENERIC_BINDING_ROUTINE,
    2 pfnUnbind type GENERIC_UNBIND_ROUTINE;
 define alias PGENERIC_BINDING_INFO handle GENERIC_BINDING_INFO;

 /* typedef EXPR_EVAL - see above typedefs for xmit_as */

 define alias XMIT_HELPER_ROUTINE
        entry (type MIDL_STUB_MESSAGE byaddr)
        limited #_RPC_USER;

 define structure
   1 XMIT_ROUTINE_QUINTUPLE,
     2 pfnTranslateToXmit   type XMIT_HELPER_ROUTINE,
     2 pfnTranslateFromXmit type XMIT_HELPER_ROUTINE,
     2 pfnFreeXmit          type XMIT_HELPER_ROUTINE,
     2 pfnFreeInst          type XMIT_HELPER_ROUTINE;
 define alias PXMIT_ROUTINE_QUINTUPLE handle XMIT_ROUTINE_QUINTUPLE;

 define alias USER_MARSHAL_SIZING_ROUTINE
        entry (fixed bin (31) byaddr,
               fixed bin(31),
               pointer)
        returns (fixed bin(31) byvalue optional)
        limited #_RPC_USER;

 define alias USER_MARSHAL_MARSHALLING_ROUTINE
        entry (fixed bin (31) byaddr,
               pointer,
               pointer)
        returns (pointer byvalue optional)
        limited #_RPC_USER;

 define alias USER_MARSHAL_UNMARSHALLING_ROUTINE
        entry (fixed bin (31) byaddr,
               pointer,
               pointer)
        returns (pointer byvalue optional)
        limited #_RPC_USER;

 define alias USER_MARSHAL_FREEING_ROUTINE
        entry (fixed bin (31) byaddr,
               pointer)
        limited #_RPC_USER;

 define structure
   1 USER_MARSHAL_ROUTINE_QUADRUPLE,
     2 pfnBufferSize type USER_MARSHAL_SIZING_ROUTINE,
     2 pfnMarshall   type USER_MARSHAL_MARSHALLING_ROUTINE,
     2 pfnUnmarshall type USER_MARSHAL_UNMARSHALLING_ROUTINE,
     2 pfnFree       type USER_MARSHAL_FREEING_ROUTINE;

 define structure
   1 USER_MARSHAL_CB,
     2 Flags    fixed bin(31),
     2 pStubMsg handle MIDL_STUB_MESSAGE,
     2 pReserve type PFORMAT_STRING;


 USER_CALL_CTXT_MASK: proc (f) returns (fixed bin (31) byvalue)
                      options (inline reorder) internal;
   dcl f fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (f, '00ff'xn));
 end USER_CALL_CTXT_MASK;

 USER_CALL_AUX_MASK: proc (f) returns (fixed bin (31) byvalue)
                     options (inline reorder) internal;
   dcl f fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (f, 'ff00'xn));
 end USER_CALL_AUX_MASK;

 GET_USER_DATA_REP: proc (f) returns (fixed bin (31) byvalue)
                    options (inline reorder) internal;
   dcl f fixed bin (31) byvalue;
   dcl isrl builtin;
   return (isrl (f, 16));
 end GET_USER_DATA_REP;


 dcl USER_CALL_IS_ASYNC fixed bin(15) value('0100'xn);
     /* aux flag: in an [async] call */


 define structure
   1 MALLOC_FREE_STRUCT,
     2 pfnAllocate entry (type size_t)
                   returns (pointer byvalue)
                   limited #_RPC_USER,
     2 pfnFree     entry (pointer)
                   limited #_RPC_USER;

 define structure
   1 COMM_FAULT_OFFSETS,
     2 CommOffset  fixed bin(15),
     2 FaultOffset fixed bin(15);

 /* MIDL Stub Descriptor */

 define structure
 1 IMPLICIT_HANDLE_INFO union,
    2 pAutoHandle         pointer,
    2 pPrimitiveHandle    pointer,
    2 pGenericBindingInfo type PGENERIC_BINDING_INFO;

 define structure
   1 MIDL_STUB_DESC,
     2 RpcInterfaceInformation    pointer,
     2 pfnAllocate                entry (type size_t)
                                  returns (pointer byvalue)
                                  limited #_RPC_API,
     2 pfnFree                    entry (pointer)
                                  limited #_RPC_API,
     2 *                          type IMPLICIT_HANDLE_INFO,
     2 apfnNdrRundownRoutines     pointer,
     2 aGenericBindingRoutinePairs handle GENERIC_BINDING_ROUTINE_PAIR,
     2 apfnExprEval               pointer,
     2 aXmitQuintuple             handle XMIT_ROUTINE_QUINTUPLE,
     2 pFormatTypes               pointer,
     2 fCheckBounds               fixed bin(31),
     /* Ndr library version. */
     2 Version                    fixed bin(31),
     2 pMallocFreeStruct          handle MALLOC_FREE_STRUCT,
     2 MIDLVersion                fixed bin(31),
     2 CommFaultOffsets           handle COMM_FAULT_OFFSETS,
     /* New fields for version 3.0+ */
     2 aUserMarshalQuadruple      handle USER_MARSHAL_ROUTINE_QUADRUPLE,
     /* Notify routines - added for NT5, MIDL 5.0 */
     2 NotifyRoutineTable         pointer,
     /* Reserved for future use. */
     2 Reserved2                  type ULONG_PTR,
     2 Reserved3                  type ULONG_PTR,
     2 Reserved4                  type ULONG_PTR,
     2 Reserved5                  type ULONG_PTR;

 define alias PMIDL_STUB_DESC handle MIDL_STUB_DESC;

 define alias PMIDL_XMIT_TYPE pointer;

 /* MIDL Stub Format String.  This is a const in the stub. */
 define structure
   1 MIDL_FORMAT_STRING,
     2 Pad    fixed bin(15),
     2 Format char(0) varz; /* real length unknown */

 /* Stub thunk used for some interpreted server stubs. */

 define alias STUB_THUNK
        entry (type MIDL_STUB_MESSAGE byaddr)
        limited #_RPC_API;

 define alias SERVER_ROUTINE
        entry ()
        returns (fixed bin(31) byvalue optional)
        limited #_RPC_API;

 /* Server Interpreter's information strucuture. */
 define structure
   1 MIDL_SERVER_INFO,
     2 pStubDesc            type PMIDL_STUB_DESC,
     2 DispatchTable        pointer,
     2 ProcString           type PFORMAT_STRING,
     2 FmtStringOffset      pointer,
     2 ThunkTable           pointer,
     2 LocalFormatTypes     type PFORMAT_STRING,
     2 LocalProcString      type PFORMAT_STRING,
     2 LocalFmtStringOffset pointer;
 define alias PMIDL_SERVER_INFO handle MIDL_SERVER_INFO;

 /* Stubless object proxy information structure. */
 define structure
   1 MIDL_STUBLESS_PROXY_INFO,
     2 pStubDesc            type PMIDL_STUB_DESC,
     2 ProcFormatString     type PFORMAT_STRING,
     2 FormatStringOffset   pointer,
     2 LocalFormatTypes     type PFORMAT_STRING,
     2 LocalProcString      type PFORMAT_STRING,
     2 LocalFmtStringOffset pointer;
 define alias PMIDL_STUBLESS_PROXY_INFO type MIDL_STUBLESS_PROXY_INFO;

 /* This is the return value from NdrClientCall. */
 define structure
   1 CLIENT_CALL_RETURN union,
     2 Pointer pointer,
     2 Simple  type LONG_PTR;

 /* Full pointer data structures. */

 define ordinal XLAT_SIDE
   ( XLAT_SERVER value(1),
     XLAT_CLIENT
   ) prec(31);

 /* Stores the translation for the conversion from a full pointer */
 /* into its corresponding ref id.                                */
 define structure
   1 FULL_PTR_TO_REFID_ELEMENT,
     2 Next    handle FULL_PTR_TO_REFID_ELEMENT,
     2 Pointer pointer,
     2 RefId   fixed bin(31),
     2 State   char;
 define alias PFULL_PTR_TO_REFID_ELEMENT
        handle FULL_PTR_TO_REFID_ELEMENT;

 /* Full pointer translation tables. */

 define structure
   1 FULL_PTR_XLAT_TABLES,
    /* Ref id to pointer translation information. */
    2 RefIdToPointer,
      3 XlatTable       pointer,
      3 StateTable      pointer,
      3 NumberOfEntries fixed bin(31),
    /* Pointer to ref id translation information. */
    2 PointerToRefId,
      3 XlatTable       pointer,
      3 NumberOfBuckets fixed bin(31),
      3 HashMask        fixed bin(31),
    /* Next ref id to use. */
    2 NextRefId         fixed bin(31),
    /* Keep track of the translation size we're handling : server or  */
    /* client. This tells us when we have to do reverse translations  */
    /* when we insert new translations.  On the server we must insert */
    /* a pointer-to-refid translation whenever we insert a            */
    /* refid-to-pointer translation, and vica versa for the client.   */
    2 XlatSide          type XLAT_SIDE;
 define alias PFULL_PTR_XLAT_TABLES handle FULL_PTR_XLAT_TABLES;

 /*********************************************************************/
 /* New MIDL 2.0 Ndr routine templates                                */
 /*********************************************************************/

 /* Marshall routines */

 dcl NdrSimpleTypeMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            char)
     RPCRTAPI RPC_ENTRY external('NdrSimpleTypeMarshall');

 dcl NdrPointerMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrPointerMarshall');

 /* Structures */

 dcl NdrSimpleStructMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSimpleStructMarshall');

 dcl NdrConformantStructMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStructMarshall');

 dcl NdrConformantVaryingStructMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingStructMarshall');

 dcl NdrHardStructMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrHardStructMarshall');

 dcl NdrComplexStructMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexStructMarshall');

 /* Arrays */

 dcl NdrFixedArrayMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFixedArrayMarshall');

 dcl NdrConformantArrayMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantArrayMarshall');

 dcl NdrConformantVaryingArrayMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingArrayMarshall');

 dcl NdrVaryingArrayMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrVaryingArrayMarshall');

 dcl NdrComplexArrayMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexArrayMarshall');

 /* Strings */

 dcl NdrNonConformantStringMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonConformantStringMarshall');

 dcl NdrConformantStringMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStringMarshall');

 /* Unions */

 dcl NdrEncapsulatedUnionMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrEncapsulatedUnionMarshall');

 dcl NdrNonEncapsulatedUnionMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonEncapsulatedUnionMarshall');

 /* Byte count pointer */

 dcl NdrByteCountPointerMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrByteCountPointerMarshall');

 /* Transmit as and represent as*/

 dcl NdrXmitOrRepAsMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrXmitOrRepAsMarshall');

 /* User_marshal */

 dcl NdrUserMarshalMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalMarshall');

 /* Cairo interface pointer */

 dcl NdrInterfacePointerMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrInterfacePointerMarshall');

 /* Context handles */

 dcl NdrClientContextMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type NDR_CCONTEXT,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrClientContextMarshall');

 dcl NdrServerContextMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type NDR_SCONTEXT,
            type NDR_RUNDOWN)
     RPCRTAPI RPC_ENTRY external('NdrServerContextMarshall');

 dcl NdrServerContextNewMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type NDR_SCONTEXT,
            type NDR_RUNDOWN,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrServerContextNewMarshall');

 /* Unmarshall routines */

 dcl NdrSimpleTypeUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            char)
     RPCRTAPI RPC_ENTRY external('NdrSimpleTypeUnmarshall');

 dcl NdrRangeUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrRangeUnmarshall');

 dcl NdrCorrelationInitialize
     entry (type MIDL_STUB_MESSAGE byaddr,
            fixed bin (31) byaddr,
            fixed bin(31),
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrCorrelationInitialize');

 dcl NdrCorrelationPass
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrCorrelationPass');

 dcl NdrCorrelationFree
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrCorrelationFree');

 dcl NdrPointerUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrPointerUnmarshall');

 /* Structures */

 dcl NdrSimpleStructUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSimpleStructUnmarshall');

 dcl NdrConformantStructUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStructUnmarshall');

 dcl NdrConformantVaryingStructUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY
     external('NdrConformantVaryingStructUnmarshall');

 dcl NdrHardStructUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrHardStructUnmarshall');

 dcl NdrComplexStructUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexStructUnmarshall');

 /* Arrays */

 dcl NdrFixedArrayUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFixedArrayUnmarshall');

 dcl NdrConformantArrayUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantArrayUnmarshall');

 dcl NdrConformantVaryingArrayUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingArrayUnmarshall');

 dcl NdrVaryingArrayUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrVaryingArrayUnmarshall');

 dcl NdrComplexArrayUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexArrayUnmarshall');

 /* Strings */

 dcl NdrNonConformantStringUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonConformantStringUnmarshall');

 dcl NdrConformantStringUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStringUnmarshall');

 /* Unions */

 dcl NdrEncapsulatedUnionUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrEncapsulatedUnionUnmarshall');

 dcl NdrNonEncapsulatedUnionUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonEncapsulatedUnionUnmarshall');

 /* Byte count pointer */

 dcl NdrByteCountPointerUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrByteCountPointerUnmarshall');

 /* Transmit as and represent as*/

 dcl NdrXmitOrRepAsUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrXmitOrRepAsUnmarshall');

 /* User_marshal */

 dcl NdrUserMarshalUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalUnmarshall');

 /* Cairo interface pointer */

 dcl NdrInterfacePointerUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type PFORMAT_STRING,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrInterfacePointerUnmarshall');

 /* Context handles */

 dcl NdrClientContextUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type NDR_CCONTEXT byaddr,
            type RPC_BINDING_HANDLE)
     RPCRTAPI RPC_ENTRY external('NdrClientContextUnmarshall');

 dcl NdrServerContextUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr)
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerContextUnmarshall');

 /* New context handle flavors */

 dcl NdrContextHandleInitialize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrContextHandleInitialize');

 dcl NdrServerContextNewUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (type NDR_SCONTEXT byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerContextNewUnmarshall');

 /* Buffer sizing routines */

 dcl NdrPointerBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrPointerBufferSize');

 /* Structures */

 dcl NdrSimpleStructBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrSimpleStructBufferSize');

 dcl NdrConformantStructBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantStructBufferSize');

 dcl NdrConformantVaryingStructBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY
     external('NdrConformantVaryingStructBufferSize');

 dcl NdrHardStructBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrHardStructBufferSize');

 dcl NdrComplexStructBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrComplexStructBufferSize');

 /* Arrays */

 dcl NdrFixedArrayBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrFixedArrayBufferSize');

 dcl NdrConformantArrayBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantArrayBufferSize');

 dcl NdrConformantVaryingArrayBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingArrayBufferSize');

 dcl NdrVaryingArrayBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrVaryingArrayBufferSize');

 dcl NdrComplexArrayBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrComplexArrayBufferSize');

 /* Strings */

 dcl NdrConformantStringBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantStringBufferSize');

 dcl NdrNonConformantStringBufferSize
      entry (type MIDL_STUB_MESSAGE byaddr,
             pointer,
             type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrNonConformantStringBufferSize');

 /* Unions */

 dcl NdrEncapsulatedUnionBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrEncapsulatedUnionBufferSize');

 dcl NdrNonEncapsulatedUnionBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrNonEncapsulatedUnionBufferSize');

 /* Byte count pointer */

 dcl NdrByteCountPointerBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrByteCountPointerBufferSize');

 /* Transmit as and represent as*/

 dcl NdrXmitOrRepAsBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrXmitOrRepAsBufferSize');

 /* User_marshal */

 dcl NdrUserMarshalBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalBufferSize');

 /* Cairo Interface pointer */

 dcl NdrInterfacePointerBufferSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrInterfacePointerBufferSize');

 /* Context Handle size */

 dcl NdrContextHandleSize
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrContextHandleSize');

 /* Memory sizing routines */

 dcl NdrPointerMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrPointerMemorySize');

 /* Structures */

 dcl NdrSimpleStructMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSimpleStructMemorySize');

 dcl NdrConformantStructMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStructMemorySize');

 dcl NdrConformantVaryingStructMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY
     external('NdrConformantVaryingStructMemorySize');

 dcl NdrHardStructMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrHardStructMemorySize');

 dcl NdrComplexStructMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexStructMemorySize');

 /* Arrays */

 dcl NdrFixedArrayMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFixedArrayMemorySize');

 dcl NdrConformantArrayMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantArrayMemorySize');

 dcl NdrConformantVaryingArrayMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingArrayMemorySize');

 dcl NdrVaryingArrayMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrVaryingArrayMemorySize');

 dcl NdrComplexArrayMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrComplexArrayMemorySize');

 /* Strings */

 dcl NdrConformantStringMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrConformantStringMemorySize');

 dcl NdrNonConformantStringMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonConformantStringMemorySize');

 /* Unions */

 dcl NdrEncapsulatedUnionMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrEncapsulatedUnionMemorySize');

 dcl NdrNonEncapsulatedUnionMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNonEncapsulatedUnionMemorySize');

 /* Transmit as and represent as*/

 dcl NdrXmitOrRepAsMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrXmitOrRepAsMemorySize');

 /* User_marshal */

 dcl NdrUserMarshalMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalMemorySize');

 /* Cairo Interface pointer */

 dcl NdrInterfacePointerMemorySize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrInterfacePointerMemorySize');

 /* Freeing routines */

 dcl NdrPointerFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrPointerFree');

 /* Structures */

 dcl NdrSimpleStructFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrSimpleStructFree');

 dcl NdrConformantStructFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantStructFree');

 dcl NdrConformantVaryingStructFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingStructFree');

 dcl NdrHardStructFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrHardStructFree');

 dcl NdrComplexStructFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrComplexStructFree');

 /* Arrays */

 dcl NdrFixedArrayFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrFixedArrayFree');

 dcl NdrConformantArrayFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantArrayFree');

 dcl NdrConformantVaryingArrayFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConformantVaryingArrayFree');

 dcl NdrVaryingArrayFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrVaryingArrayFree');

 dcl NdrComplexArrayFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrComplexArrayFree');

 /* Unions */

 dcl NdrEncapsulatedUnionFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrEncapsulatedUnionFree');

 dcl NdrNonEncapsulatedUnionFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrNonEncapsulatedUnionFree');

 /* Byte count */

 dcl NdrByteCountPointerFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrByteCountPointerFree');

 /* Transmit as and represent as*/

 dcl NdrXmitOrRepAsFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrXmitOrRepAsFree');

 /* User_marshal */

 dcl NdrUserMarshalFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalFree');

 /* Cairo Interface pointer */

 dcl NdrInterfacePointerFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrInterfacePointerFree');

 /* Endian conversion routine. */

 dcl NdrConvert2
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrConvert2');

 dcl NdrConvert
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrConvert');

 dcl USER_MARSHAL_FC_BYTE   fixed bin(31) value(1);
 dcl USER_MARSHAL_FC_CHAR   fixed bin(31) value(2);
 dcl USER_MARSHAL_FC_SMALL  fixed bin(31) value(3);
 dcl USER_MARSHAL_FC_USMALL fixed bin(31) value(4);
 dcl USER_MARSHAL_FC_WCHAR  fixed bin(31) value(5);
 dcl USER_MARSHAL_FC_SHORT  fixed bin(31) value(6);
 dcl USER_MARSHAL_FC_USHORT fixed bin(31) value(7);
 dcl USER_MARSHAL_FC_LONG   fixed bin(31) value(8);
 dcl USER_MARSHAL_FC_ULONG  fixed bin(31) value(9);
 dcl USER_MARSHAL_FC_FLOAT  fixed bin(31) value(10);
 dcl USER_MARSHAL_FC_HYPER  fixed bin(31) value(11);
 dcl USER_MARSHAL_FC_DOUBLE fixed bin(31) value(12);

 dcl NdrUserMarshalSimpleTypeConvert
     entry (fixed bin(31) byaddr,
            pointer,
            char)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrUserMarshalSimpleTypeConvert');

 /* Auxilary routines */

 dcl NdrClientInitializeNew
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrClientInitializeNew');

 dcl NdrServerInitializeNew
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerInitializeNew');

 dcl NdrServerInitializePartial
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrServerInitializePartial');

 dcl NdrClientInitialize
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrClientInitialize');

 dcl NdrServerInitialize
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerInitialize');

 dcl NdrServerInitializeUnmarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr,
            type RPC_MESSAGE byaddr)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerInitializeUnmarshall');

 dcl NdrServerInitializeMarshall
     entry (type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrServerInitializeMarshall');

 dcl NdrGetBuffer
     entry (type MIDL_STUB_MESSAGE byaddr,
            fixed bin(31),
            type RPC_BINDING_HANDLE)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrGetBuffer');

 dcl NdrNsGetBuffer
     entry (type MIDL_STUB_MESSAGE byaddr,
           fixed bin(31),
            type RPC_BINDING_HANDLE)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNsGetBuffer');

 dcl NdrGetPipeBuffer
     entry (type MIDL_STUB_MESSAGE byaddr,
            fixed bin(31),
            type RPC_BINDING_HANDLE)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrGetPipeBuffer');

 dcl NdrGetPartialBuffer
     entry (type MIDL_STUB_MESSAGE byaddr )
     RPCRTAPI RPC_ENTRY external('NdrGetPartialBuffer');

 dcl NdrSendReceive
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSendReceive');

 dcl NdrNsSendReceive
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            type RPC_BINDING_HANDLE byaddr)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrNsSendReceive');

 dcl NdrPipeSendReceive
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PNDR_PIPE_DESC)
     RPCRTAPI RPC_ENTRY external('NdrPipeSendReceive');

 dcl NdrFreeBuffer
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrFreeBuffer');

 dcl NdrGetDcomProtocolVersion
     entry (type MIDL_STUB_MESSAGE byaddr,
            type RPC_VERSION byaddr)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrGetDcomProtocolVersion');


 /* Pipe specific calls */

 dcl NdrPipesInitialize
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING,
            type PNDR_PIPE_DESC,
            type PNDR_PIPE_MESSAGE,
            pointer,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrPipesInitialize');

 dcl NdrPipePull  entry (
            pointer,
            pointer,
            fixed bin(31),
            fixed bin(31) byaddr)
     RPCRTAPI RPC_ENTRY external('NdrPipePull');

 dcl NdrPipePush
     entry (pointer,
            pointer,
            fixed bin(31))
     RPCRTAPI RPC_ENTRY external('NdrPipePush');

 dcl NdrIsAppDoneWithPipes
     entry (type PNDR_PIPE_DESC)
     RPCRTAPI RPC_ENTRY external('NdrIsAppDoneWithPipes');

 dcl NdrPipesDone
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrPipesDone');


 /* Interpeter calls. */

 /* client */
 /*

 dcl NdrClientCall2
     entry (type MIDL_STUB_DESC byaddr,
            type PFORMAT_STRING list)
     returns (type CLIENT_CALL_RETURN byvalue optional)
     RPC_VAR_ENTRY external('NdrClientCall2');

 dcl NdrClientCall
     entry (type MIDL_STUB_DESC byaddr,
            type PFORMAT_STRING list)
     returns (type CLIENT_CALL_RETURN byvalue optional)
     RPC_VAR_ENTRY external('NdrClientCall');

 dcl NdrAsyncClientCall
     entry (type MIDL_STUB_DESC byaddr,
            type PFORMAT_STRING list)
     returns (type CLIENT_CALL_RETURN byvalue optional)
     RPC_VAR_ENTRY external('NdrAsyncClientCall');

 dcl NdrDcomAsyncClientCall
     entry (type MIDL_STUB_DESC byaddr,
            type PFORMAT_STRING list)
     returns (type CLIENT_CALL_RETURN byvalue optional)
     RPC_VAR_ENTRY external('NdrDcomAsyncClientCall');
 */

 /* server */
 define ordinal STUB_PHASE
   ( STUB_UNMARSHAL,
     STUB_CALL_SERVER,
     STUB_MARSHAL,
     STUB_CALL_SERVER_NO_HRESULT
   ) prec(31);


 define ordinal PROXY_PHASE
   ( PROXY_CALCSIZE,
     PROXY_GETBUFFER,
     PROXY_MARSHAL,
     PROXY_SENDRECEIVE,
     PROXY_UNMARSHAL
   ) prec(31);

 define structure 1 IRpcStubBuffer;

 /* Raw RPC only */
 dcl NdrAsyncServerCall
     entry (type RPC_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrAsyncServerCall');

 /* old dcom async scheme */
 dcl NdrAsyncStubCall
     entry (handle IRpcStubBuffer,
            handle IRpcChannelBuffer,
            type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrAsyncStubCall');

 /* async uuid */
 dcl NdrDcomAsyncStubCall
     entry (handle IRpcStubBuffer,
            handle IRpcChannelBuffer,
            type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrDcomAsyncStubCall');

 dcl NdrStubCall2
     entry (handle IRpcStubBuffer,
            handle IRpcChannelBuffer,
            type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrStubCall2');

 dcl NdrServerCall2
     entry (type RPC_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrServerCall2');

 dcl NdrStubCall
     entry (handle IRpcStubBuffer,
            handle IRpcChannelBuffer,
            type RPC_MESSAGE byaddr,
            fixed bin(31) byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrStubCall');

 dcl NdrServerCall
     entry (type RPC_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrServerCall');

 dcl NdrServerUnmarshall
     entry (handle IRpcChannelBuffer,
            type RPC_MESSAGE byaddr,
            type MIDL_STUB_MESSAGE byaddr,
            type MIDL_STUB_DESC byaddr,
            type PFORMAT_STRING,
            pointer)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrServerUnmarshall');

 dcl NdrServerMarshall
     entry (handle IRpcStubBuffer,
            handle IRpcChannelBuffer,
            type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING)
     RPCRTAPI RPC_ENTRY external('NdrServerMarshall');

 /* Comm and Fault status */

 dcl NdrMapCommAndFaultStatus
     entry (type MIDL_STUB_MESSAGE byaddr,
            fixed bin(31) byaddr,
            fixed bin(31) byaddr,
            type RPC_STATUS)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrMapCommAndFaultStatus');

 /* Helper routines */

 dcl NdrSH_UPDecision
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            type RPC_BUFPTR)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_UPDecision');

 dcl NdrSH_TLUPDecision
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_TLUPDecision');

 dcl NdrSH_TLUPDecisionBuffer
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_TLUPDecisionBuffer');

 dcl NdrSH_IfAlloc
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfAlloc');

 dcl NdrSH_IfAllocRef
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfAllocRef');

 dcl NdrSH_IfAllocSet
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfAllocSet');

 dcl NdrSH_IfCopy
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (type RPC_BUFPTR byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfCopy');

 dcl NdrSH_IfAllocCopy
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (type RPC_BUFPTR byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfAllocCopy');

 dcl NdrSH_Copy
     entry (pointer,
            pointer,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_Copy');

 dcl NdrSH_IfFree
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer)
     RPCRTAPI RPC_ENTRY external('NdrSH_IfFree');


 dcl NdrSH_StringMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer,
            fixed bin(31),
            fixed bin(31))
     returns (type RPC_BUFPTR byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_StringMarshall');

 dcl NdrSH_StringUnMarshall
     entry (type MIDL_STUB_MESSAGE byaddr,
            pointer byaddr,
            fixed bin(31))
     returns (type RPC_BUFPTR byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrSH_StringUnMarshall');

 /*********************************************************************/
 /*   MIDL 2.0 memory package: rpc_ss_* rpc_sm_*                      */
 /*********************************************************************/

 define alias RPC_SS_THREAD_HANDLE pointer;

 define alias RPC_CLIENT_ALLOC
        entry (type size_t)
        returns (pointer byvalue optional)
        limited #_RPC_API;

 define alias RPC_CLIENT_FREE
        entry (pointer)
        limited #_RPC_API;

 /* ++                */
 /*    RpcSs* package */
 /* --                */

 dcl RpcSsAllocate
     entry (type size_t)
     returns (pointer byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSsAllocate');

 dcl RpcSsDisableAllocate
     entry ()
     RPCRTAPI RPC_ENTRY external('RpcSsDisableAllocate');

 dcl RpcSsEnableAllocate
     entry ()
     RPCRTAPI RPC_ENTRY external('RpcSsEnableAllocate');

 dcl RpcSsFree
     entry (pointer)
     RPCRTAPI RPC_ENTRY external('RpcSsFree');

 dcl RpcSsGetThreadHandle
     entry ()
     returns (type RPC_SS_THREAD_HANDLE byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSsGetThreadHandle');

 dcl RpcSsSetClientAllocFree
     entry (type RPC_CLIENT_ALLOC,
            type RPC_CLIENT_FREE)
     RPCRTAPI RPC_ENTRY external('RpcSsSetClientAllocFree');

 dcl RpcSsSetThreadHandle
     entry (type RPC_SS_THREAD_HANDLE)
     RPCRTAPI RPC_ENTRY external('RpcSsSetThreadHandle');

 dcl RpcSsSwapClientAllocFree
     entry (type RPC_CLIENT_ALLOC,
            type RPC_CLIENT_FREE,
            type RPC_CLIENT_ALLOC byaddr,
            type RPC_CLIENT_FREE byaddr)
     RPCRTAPI RPC_ENTRY external('RpcSsSwapClientAllocFree');

 /* ++                */
 /*    RpcSm* package */
 /* --                */

 dcl RpcSmAllocate
     entry (type size_t,
            type RPC_STATUS byaddr)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmAllocate');

 dcl RpcSmClientFree
     entry (pointer)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmClientFree');

 dcl RpcSmDestroyClientContext
     entry (pointer byaddr)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmDestroyClientContext');

 dcl RpcSmDisableAllocate
     entry ()
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmDisableAllocate');

 dcl RpcSmEnableAllocate
     entry ()
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmEnableAllocate');

 dcl RpcSmFree
     entry (pointer)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmFree');

 dcl RpcSmGetThreadHandle
     entry (type RPC_STATUS byaddr)
     returns (type RPC_SS_THREAD_HANDLE byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmGetThreadHandle');

 dcl RpcSmSetClientAllocFree
     entry (type RPC_CLIENT_ALLOC,
            type RPC_CLIENT_FREE)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmSetClientAllocFree');

 dcl RpcSmSetThreadHandle
     entry (type RPC_SS_THREAD_HANDLE)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmSetThreadHandle');

 dcl RpcSmSwapClientAllocFree
     entry (type RPC_CLIENT_ALLOC,
            type RPC_CLIENT_FREE,
            type RPC_CLIENT_ALLOC byaddr,
            type RPC_CLIENT_FREE byaddr)
     returns (type RPC_STATUS byvalue optional)
     RPCRTAPI RPC_ENTRY external('RpcSmSwapClientAllocFree');

 /* ++                       */
 /*    Ndr stub entry points */
 /* --                       */

 dcl NdrRpcSsEnableAllocate
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrRpcSsEnableAllocate');

 dcl NdrRpcSsDisableAllocate
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrRpcSsDisableAllocate');

 dcl NdrRpcSmSetClientToOsf
     entry (type MIDL_STUB_MESSAGE byaddr)
     RPCRTAPI RPC_ENTRY external('NdrRpcSmSetClientToOsf');

 dcl NdrRpcSmClientAllocate
     entry (type size_t)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrRpcSmClientAllocate');

 dcl NdrRpcSmClientFree
     entry (pointer)
     RPCRTAPI RPC_ENTRY external('NdrRpcSmClientFree');

 dcl NdrRpcSsDefaultAllocate
     entry (type size_t)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrRpcSsDefaultAllocate');

 dcl NdrRpcSsDefaultFree
     entry (pointer)
     RPCRTAPI RPC_ENTRY external('NdrRpcSsDefaultFree');

 /*********************************************************************/
 /*   end of memory package: rpc_ss_* rpc_sm_*                        */
 /*********************************************************************/

 /*********************************************************************/
 /* Full Pointer APIs                                                 */
 /*********************************************************************/

 dcl NdrFullPointerXlatInit
     entry (fixed bin(31),
            type XLAT_SIDE)
     returns (type PFULL_PTR_XLAT_TABLES byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerXlatInit');

 dcl NdrFullPointerXlatFree
     entry (type PFULL_PTR_XLAT_TABLES)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerXlatFree');

 dcl NdrFullPointerQueryPointer
     entry (type PFULL_PTR_XLAT_TABLES,
            pointer,
            char,
            fixed bin(31) byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerQueryPointer');

 dcl NdrFullPointerQueryRefId
     entry (type PFULL_PTR_XLAT_TABLES,
            fixed bin(31),
            char,
            pointer byaddr)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerQueryRefId');

 dcl NdrFullPointerInsertRefId
     entry (type PFULL_PTR_XLAT_TABLES,
            fixed bin(31),
            pointer)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerInsertRefId');

 dcl NdrFullPointerFree
     entry (type PFULL_PTR_XLAT_TABLES,
            pointer)
     returns (fixed bin(31) byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrFullPointerFree');

 dcl NdrAllocate
     entry (type MIDL_STUB_MESSAGE byaddr,
            type size_t)
     returns (ptr byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrAllocate');

 dcl NdrClearOutParameters
     entry (type MIDL_STUB_MESSAGE byaddr,
            type PFORMAT_STRING,
            pointer)
     RPCRTAPI RPC_ENTRY external('NdrClearOutParameters');


 /*********************************************************************/
 /* Proxy APIs                                                        */
 /*********************************************************************/

 dcl NdrOleAllocate
     entry (type size_t)
     returns (pointer byvalue optional)
     RPCRTAPI RPC_ENTRY external('NdrOleAllocate');

 dcl NdrOleFree
     entry (pointer)
     RPCRTAPI RPC_ENTRY external('NdrOleFree');

 %dcl CONST_VTBL char ext;

 %dcl CONST_VTABLE char ext;
 %if CONST_VTABLE ^= '' %then %do;
   %CONST_VTBL = ''; /* sort out when know how */
 %end; %else %do;
   %CONST_VTBL = '';
 %end;

 /*********************************************************************/
 /* Special things for VC5 Com support                                */
 /*********************************************************************/

 /* change when PL/I supports uuid declarations for DCOM */
 %DECLSPEC_UUID: proc (x) returns (char);
   dcl x char;
   return ('');
 %end DECLSPEC_UUID;
 %dcl DECLSPEC_UUID entry;

 %MIDL_INTERFACE: proc (x) returns (char);
   dcl x char;
   return ('define structure ');
 %end MIDL_INTERFACE;
 %dcl MIDL_INTERFACE entry;


 %EXTERN_GUID: proc (itf,l1,s1,s2,c1,c2,c3,c4,c5,c6,c7,c8)
               returns (char);
   dcl (itf,l1,s1,s2,c1,c2,c3,c4,c5,c6,c7,c8) char;
   /* When structure INITs are supported, replace with: */
   /* return ('dcl ' || itf || ' type IID static external init (' || */
   /*            l1 || ',' || s1 || ',' || s2 || ', (' ||            */
   /*            c1 || ',' || c2 || ',' || c3 || ',' || c4 || ',' || */
   /*            c5 || ',' || c6 || ',' || c7 || ',' || c8 || '))';  */
   /* In the meantime we have to use: */
   return ('dcl 1 ' || itf || ' static external, ' ||
                  '2 Data1 type DWORD init (' || l1 || '), ' ||
                  '2 Data2 type  WORD init (' || s1 || '), ' ||
                  '2 Data3 type  WORD init (' || s2 || '), ' ||
                  '2 Data4 (8)   char init (' ||
            c1 || ',' || c2 || ',' || c3 || ',' || c4 || ',' ||
            c5 || ',' || c6 || ',' || c7 || ',' || c8 || ')');
 %end EXTERN_GUID;
 %dcl EXTERN_GUID entry;

 %end; /* __RPCNDR_H__ */


