 /*********************************************************************/
 /*                                                                   */
 /*  PL/I Include file: SQLUDF.CPY                                    */
 /*                                                                   */
 /*    This file is installed in the \IBMPLIW\INCLUDE\ directory.     */
 /*                                                                   */
 /*  NAME - SQLUDF.CPY                                                */
 /*                                                                   */
 /*  DESCRIPTION                                                      */
 /*                                                                   */
 /*    This include file is translated into a PL/I include file,      */
 /*    from the C header files supplied with the                      */
 /*    IBM DATABASE 2 SOFTWARE DEVELOPERS KIT/2  product.             */
 /*    This include file is intended for use with the                 */
 /*    IBM DATABASE 2 SOFTWARE DEVELOPERS KIT/2 product.              */
 /*                                                                   */
 /*                                                                   */
 /*    (C) Copyright IBM Corp. 1994, 1996. All Rights Reserved.       */
 /*    US Government Users Restricted Rights-- Use, duplication or    */
 /*    disclosure restricted by GSA ADP Schedule Contract with        */
 /*    IBM Corp.                                                      */
 /*    Licensed Materials - Property of IBM                           */
 /*                                                                   */
 /*Function = Include File for use in compiling User Defined Function */
 /*          Intended to help writers of UDFs deal with the interface */
 /*             between DBMS and UDF.  Contains:                      */
 /*               common structures                                   */
 /*               "language types" for the SQL types                  */
 /*               enumerated type declaration for call-type argument  */
 /*               SQLSTATE constants                                  */
 /*               macro for prescribed part of the argument list      */
 /*               macro for scratchpad oriented part of argument list */
 /*              macros to address prescribed parts of argument list  */
 /*              macros to test for argument null and notnull         */
 /*                                                                   */
 /*  DISCLAIMER OF WARRANTIES                                         */
 /*    The following enclosed code is sample code created by IBM    */
 /*    Corporation. This sample code is not part of any standard      */
 /*    IBM product and is provided to you solely for the purpose of   */
 /*    assisting you in the development of your applications.  The    */
 /*    code is provided "AS IS", without warranty of any kind.        */
 /*    IBM shall not be liable for any damages arising out of your    */
 /*    use of the sample code, even if IBM has been advised of the    */
 /*    possibility of such damages.                                   */
 /*                                                                   */
 /*********************************************************************/

 %dcl SQL_H_SQLUDF char ext;
 %if SQL_H_SQLUDF = '' %then %do;
 /* Permit duplicate Includes */
 %SQL_H_SQLUDF = 'Y';

 %dcl DB2390 char ext;

 /* System-Specific Include   */
 %dcl SQL_H_SQLSYSTM char ext;
 %if SQL_H_SQLSYSTM = '' %then %do;
 %include sqlsystm;
 %end;


 /*==============================================*/
 /*== Common  dcls, used in this file.         ==*/
 /*==============================================*/

 /*********************************************************************/
 /* When the following LEN defines are used to define a nul-terminated*/
 /* string, be sure to "add 1" for the null terminating character.    */
 /*********************************************************************/
 dcl SQLUDF_SQLSTATE_LEN         fixed bin(31) value(5);
                                 /* length of SQLSTATE                */
 dcl SQLUDF_SCRATCHPAD_LEN       fixed bin(31) value(100);
                                 /* length of scratchpad              */
 dcl SQLUDF_FQNAME_LEN           fixed bin(31) value(517);
                                 /* length of fully-qualified name    */
 dcl SQLUDF_SPECNAME_LEN         fixed bin(31) value(128);
                                 /* length of function specific name  */
 dcl SQLUDF_MSGTEXT_LEN          fixed bin(31) value(70);
                                 /* length of message text area       */
 dcl SQLUDF_MAX_IDENT_LEN        fixed bin(31) value(128 );
                                 /* max length of identifier          */
 dcl SQLUDF_SH_IDENT_LEN         fixed bin(31) value(8);
                                 /* length of short identifier        */

 /*********************************************************************/
 /* LIMIT on total number of columns for a result table of TABLE      */
 /* FUNCTION.                                                         */
 /* This may be DIFFERENT than the regular limit on DB2 tables.       */
 /*********************************************************************/
 %if DB2390 ^= '' %then %do;
 dcl SQLUDF_MAXCOL_TABFUN        fixed bin(31) value(750);
 %end; %else %do;
 dcl SQLUDF_MAXCOL_TABFUN        fixed bin(31) value(255);
 %end;


 /*=================================================*/
 /*== Common structures.                          ==*/
 /*== Note:  these are used in the PL/I "language ==*/
 /*==        type" definitions which follow.      ==*/
 /*=================================================*/

 /*-------------------------------------------------------------------*/
 /* NOTE: The following defined structures are "models" for the user. */
 /*       The user must replace the '1' with lengths that are         */
 /*       meaningful to their applications.                           */
 /*-------------------------------------------------------------------*/


 /*------------------------------------------------------------------*/
 /* Structure used for VARCHAR FOR BIT DATA, LONG VARCHAR            */
 /* sqludf_vc_fbd is actually just a map for char (???) var          */
 /*------------------------------------------------------------------*/
 define structure
   1 sqludf_vc_fbd,
     2 length unsigned fixed bin(16),      /* length of data */
     2 data   (1) char(1);                 /* REFER */


 /*------------------------------------------------------------------*/
 /* Structure used for LONG VARGRAPHIC                               */
 /* sqludf_vg is actually just a map for graphic (???) var           */
 /*------------------------------------------------------------------*/
 define structure
   1 sqludf_vg,
     2 length unsigned fixed bin(16),      /* length of data */
     2 data   (1) graphic(1);              /* REFER */


 /*--------------------------------*/
 /* Structure used for BLOB, CLOB  */
 /*--------------------------------*/
 define structure
   1 sqludf_lob,
     2 length fixed bin(32) unsigned,      /* length of data */
     2 data   (1) char(1);                 /* REFER */


 /*---------------------------*/
 /* Structure used for DBCLOB */
 /*---------------------------*/
 define structure
   1 sqludf_lobg,
     2 length fixed bin(32) unsigned,      /* length of data */
     2 data   (1) graphic(1);              /* REFER */

 define structure
   1 sqludf_datalink,
     2 version          fixed bin (32) unsigned,
     2 linktype         char (4),
     2 url_length       fixed bin (32) unsigned,
     2 comment_length   fixed bin (32) unsigned,
     2 reserve2         char(8),                 /* Future */
     2 url_plus_comment char(230);

 /*------------------------------------------------------------*/
 /* Structure used for: the scratchpad.                        */
 /* Note: This structure as provided by DB2/CS gives the       */
 /*       overall size/structure of the scratchpad argument.   */
 /*       Individual UDFs which use a scratchpad may wish to   */
 /*       override the "data" variable of this scratchpad      */
 /*       with variables that make sense for the particular    */
 /*       UDF.                                                 */
 /* Note: Remember that the "data" variable is initialized     */
 /*       by DB2/CS to all \0 before the first call.           */
 /*------------------------------------------------------------*/
 define structure
   1 sqludf_scratchpad,
       2 length fixed bin(32) unsigned,/* length of scratchpad data */
       2 data (SQLUDF_SCRATCHPAD_LEN) char(1);
                                 /* scratchpad data, init. to all \0 */

 /* Structure used for: Database code page.                           */
 /* DB2/CS code page structure                                        */
 define structure
   1 db2_cs_cdpg,
     2 sbcs  fixed bin(32) unsigned,   /* DB single byte CS           */
     2 dbcs  fixed bin(32) unsigned,   /* DB double byte CS           */
     2 comp  fixed bin(32) unsigned,   /* DB composite codepage CS    */
     2 dummy char(36);                 /* reserved for later use      */

 define structure
   1 db2_mvs_cdpg,
     2 ascii_sbcs      fixed bin(32) unsigned,
     2 ascii_dbcs      fixed bin(32) unsigned,
     2 ascii_mixed     fixed bin(32) unsigned,
     2 ebcdic_sbcd     fixed bin(32) unsigned,
     2 ebcdic_dbcs     fixed bin(32) unsigned,
     2 ebcdic_mixed    fixed bin(32) unsigned,
     2 encoding_scheme fixed bin(32) unsigned,
     2 reserved        char(20);


 /* Code Page Structure                                               */
 define structure
   1 db_cdpg union,
     2 cdpg_cs  type db2_cs_cdpg,   /* DB2/CS code page               */
     2 cdpg_mvs type db2_mvs_cdpg;  /* DB2 for OS/390 code page       */


 /*********************************************************************/
 /* Operating System "platform" definitions.                          */
 /*********************************************************************/
 dcl SQLUDF_PLATFORM_UNKNOWN        fixed bin(31) value(0);
                                    /* Unknown platform               */
 dcl SQLUDF_PLATFORM_OS2            fixed bin(31) value(1);
                                    /* OS/2                           */
 dcl SQLUDF_PLATFORM_WINDOWS        fixed bin(31) value(3);
                                    /* Windows                        */
 dcl SQLUDF_PLATFORM_AIX            fixed bin(31) value(4);
                                    /* AIX                            */
 dcl SQLUDF_PLATFORM_NT             fixed bin(31) value(5);
                                    /* NT                             */
 dcl SQLUDF_PLATFORM_HP             fixed bin(31) value(6);
                                    /* HP                             */
 dcl SQLUDF_PLATFORM_SUN            fixed bin(31) value(7);
                                    /* Sun                            */
 dcl SQLUDF_PLATFORM_OS390          fixed bin(31) value(8);
                                    /* OS/390                         */
 dcl SQLUDF_PLATFORM_SNI            fixed bin(31) value(13);
                                    /* Siemens Nixdorf                */
 dcl SQLUDF_PLATFORM_WINDOWS95      fixed bin(31) value(15);
                                    /* Windows 95                     */
 dcl SQLUDF_PLATFORM_SCO            fixed bin(31) value(16);
                                    /* SCO                            */
 dcl SQLUDF_PLATFORM_LINUX          fixed bin(31) value(18);
                                    /* Linux                          */
 dcl SQLUDF_PLATFORM_DYNIX          fixed bin(31) value(19);
                                    /* DYNIX/ptx                      */
 dcl SQLUDF_PLATFORM_LINUX390       fixed bin(31) value(24);
                                    /* Linux/390                      */
 dcl SQLUDF_PLATFORM_LINUXPPC       fixed bin(31) value(27);
                                    /* Linux/PPC                      */
 dcl SQLUDF_PLATFORM_AS400          fixed bin(31) value(400);
                                    /* AS/400                         */
 dcl SQLUDF_PLATFORM_MVS            fixed bin(31)
                                    value(SQLUDF_PLATFORM_OS390);
                                    /* OS/390           */

 /*********************************************************************/
 /* Structure used for: dbinfo.                                       */
 /* Note: This structure as provided by DB2 to the UDF                */
 /*       contains additional information which may be used to        */
 /*       enable (but not require) additional capability or           */
 /*       optimizations on the part of the external function.         */
 /*       This argument is only present if the DBINFO                 */
 /*       keyword is specified in the CREATE FUNCTION                 */
 /*       statement.                                                  */
 /*                                                                   */
 /* Database version/release.  The field has the format pppvvrrm:     */
 /*   ppp -- identifies the product name                              */
 /*   vv  -- two digit rversion identifier such as '03'               */
 /*   rr  -- two digit release identifier such as '01'                */
 /*   m   -- one digit modification level identifier such as '0'      */
 /*                                                                   */
 /* OS Platform information. Please refer to Operating System         */
 /* "platform" definitions above for its definition.                  */
 /*********************************************************************/

 define structure
   1 sqludf_dbinfo,
     2 dbnamelen   fixed bin(16) unsigned,    /* database name length */
     2 dbname      char(SQLUDF_MAX_IDENT_LEN),/* database name        */
     2 authidlen   fixed bin(16) unsigned, /* authorization ID length */
     2 authid      char(SQLUDF_MAX_IDENT_LEN),/* appl authorization ID*/
     2 codepg      type db_cdpg,              /* database code page   */
     2 tbschemalen fixed bin(16) unsigned,/* table schema name length */
     2 tbschema    char(SQLUDF_MAX_IDENT_LEN),/* table schema name    */
     2 tbnamelen   fixed bin(16) unsigned,    /* table name length    */
     2 tbname      char(SQLUDF_MAX_IDENT_LEN),/* table name           */
     2 colnamelen  fixed bin(16) unsigned,    /* column name length   */
     2 colname     char(SQLUDF_MAX_IDENT_LEN),/* column name          */
     2 ver_rel     char(SQLUDF_SH_IDENT_LEN),
                                          /* database version/release */
     2 resd0       char(2),                  /* Reserved for expansion*/
     2 platform    fixed bin(32) unsigned,    /* platform             */
     2 numtfcol    fixed bin(16) unsigned,
                                /* number of entries in the TF column */
                                /* list array (tfcolumn, below)       */
     2 resd1       char(38),           /* Reserved for expansion      */
     2 tfcolumn    pointer,            /* tfcolumn will be allocated  */
                                       /* dynamically if TF is defined*/
                                       /* otherwise, this will be a   */
                                       /* null pointer.               */
     2 appl_id     pointer,            /* application identifier      */
     2 resd2       char(20);           /* Reserved for expansion      */


 /*===================================================================*/
 /*== "Language type" definitions, for argument/result values.      ==*/
 /*== Usage note: define input and output SQL arguments as          ==*/
 /*==             pointers to these types.                          ==*/
 /*===================================================================*/

 define alias SQLUDF_SMALLINT fixed bin(15);  /* for SMALLINT         */
 define alias SQLUDF_INTEGER  fixed bin(31);  /* for INTEGER          */
 define alias SQLUDF_BIGINT   fixed bin(63);  /* for BIGINT (64 bit)  */
 define alias SQLUDF_DOUBLE   float bin(53);  /* for DOUBLE PRECISION */
 define alias SQLUDF_REAL     float bin(21);  /* for REAL             */

 /* declare a SQL CHAR(n) item as CHAR(n) VARYINGZ            */
 /* declare a SQL CHARVAR(n) item as CHAR(n) VARYING          */
 /* declare a SQL VARCHAR_FBD(n) item as CHAR(n) VARYING      */
 /* declare a SQL LONG(n) item as CHAR(n) VARYING             */

 /* declare a SQL GRAPH(n) item as GRAPHIC(n) VARYINGZ        */
 /* declare a SQL VARGRAPH(n) item as GRAPHIC(n) VARYING      */
 /* declare a SQL LONGVARG(n) item as GRAPHIC(n) VARYING      */

 define alias SQLUDF_DATE     char(10) varz;
 define alias SQLUDF_TIME     char(8) varz;
 define alias SQLUDF_STAMP    char(26) varz;
 define alias SQLUDF_CLOB     type sqludf_lob;       /* for CLOB      */
 define alias SQLUDF_BLOB     type sqludf_lob;       /* for BLOB      */
 define alias SQLUDF_DBCLOB   type sqludf_lobg;      /* for DBCLOB    */
 /* SQLUDF_DATALINK == sqludf_datalink */
 define alias SQLUDF_NULLIND  fixed bin(15);   /* for null indicators */

 define alias udf_locator     fixed bin(32) unsigned;
 define alias SQLUDF_LOCATOR  fixed bin(32) unsigned;
                                                   /* for LOB LOCATOR */

 /*********************************************************************/
 /* "Language type" definitions, for scratchpad and call type         */
 /* arguments to a UDF.                                               */
 /* Usage note: define arguments as pointers                          */
 /*             to these types.                                       */
 /*********************************************************************/

 dcl SQLUDF_SCRATCHPAD type sqludf_scratchpad;     /* for scratchpad */
 /* SQLUDF_CALL_TYPE == sqludf_call_type */
 /* SQLUDF_DBINFO == sqludf_dbinfo */

 /*********************************************************************/
 /* Enumerated type, for call-type argument.                          */
 /*********************************************************************/
 /* Contains generic internal values, right below, we will map to     */
 /* these values for scalar and table function call types.            */
 define ordinal
   sqludf_call_type
     ( SQLUDF_ENUM_CALLTYPE_MINUS2  value(-2),
          /* Used in table func for FIRST call  */
       SQLUDF_ENUM_CALLTYPE_MINUS1  value(-1),
          /* Used in table func for OPEN call,  */
          /* and scalar func FIRST call         */
       SQLUDF_ENUM_CALLTYPE_ZERO    value(0),
          /* Used in table func for FETCH call, */
          /* and scalar func NORMAL call        */
       SQLUDF_ENUM_CALLTYPE_PLUS1   value(1),
          /* Used in table func for CLOSE call, */
          /* and scalar func FINAL call         */
       SQLUDF_ENUM_CALLTYPE_PLUS2   value(2),
          /* Used in table func for FINAL call, */
       SQLUDF_ENUM_CALLTYPE_PLUS255 value(255),
          /* Used in table func FINAL_CRA call  */
          /* and scalar func FINAL_CRA call     */
       SQLUDF_ENUM_CALLTYPE_END     value('7FFFFFFF'xn)
          /* Present only to force 4byte type   */
     ) prec (31);
 /*********************************************************************/
 /* Call type values to be used for Scalar Functions...               */
 /*********************************************************************/

 dcl SQLUDF_FIRST_CALL   type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_MINUS1);  /* First call, only  */
           /* made if FINAL CALL spec'd on CREATE.*/
 dcl SQLUDF_NORMAL_CALL  type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_ZERO);    /* Fetch next row.   */
 dcl SQLUDF_FINAL_CALL   type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_PLUS1);   /* Final call, only  */
           /* made if FINAL CALL spec'd on CREATE.*/
 dcl SQLUDF_FINAL_CRA    type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_PLUS255); /* Final call, only  */
           /* made if FINAL CALL spec'd, and after*/
           /* UDF abend, rollback, or commit.     */

 /*********************************************************************/
 /* Call type values to be used for Table Functions...                */
 /*********************************************************************/
 dcl SQLUDF_TF_FIRST     type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_MINUS2);  /* First call, only  */
           /* made if FINAL CALL spec'd on CREATE.*/
 dcl SQLUDF_TF_OPEN      type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_MINUS1);  /* Open table.       */
 dcl SQLUDF_TF_FETCH     type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_ZERO);    /* Fetch next row.   */
 dcl SQLUDF_TF_CLOSE     type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_PLUS1);   /* Close table.      */
 dcl SQLUDF_TF_FINAL     type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_PLUS2);   /* Final call, only  */
           /* made if FINAL CALL spec'd on CREATE.*/
 dcl SQLUDF_TF_FINAL_CRA type sqludf_call_type
     value(SQLUDF_ENUM_CALLTYPE_PLUS255); /* Final call, only  */
           /* made if FINAL CALL spec'd, and after*/
           /* UDF abend, rollback, or commit.     */

 /* The following are deprecated,             */
 /* please use the SQLUDF_TF_* forms instead  */
 dcl SQL_TF_OPEN         type sqludf_call_type
     value(SQLUDF_FIRST_CALL);  /* Open table  */
 dcl SQL_TF_FETCH        type sqludf_call_type
     value(SQLUDF_NORMAL_CALL); /* Fetch next row */
 dcl SQL_TF_CLOSE        type sqludf_call_type
     value(SQLUDF_FINAL_CALL);  /* Close table */


 /*********************************************************************/
 /* Macros for defining the always-present (trailing) input arguments:*/
 /*********************************************************************/

 /*********************************************************************/
 /* For the case WITHOUT scratchpad / final_call / dbinfo arguments:  */
 /* SQLUDF_TRAIL_ARGS                                                 */
 /* defines the always present trailing arguments-- pointers to:      */
 /*               sqlstate,                                           */
 /*               fully qualified function name,                      */
 /*               function specific name,                             */
 /*               message text                                        */
 /*                                                                   */
 /* Usage note: when both SCRATCHPAD and FINAL CALL are defined for   */
 /*             the function, use SQLUDF_TRAIL_ARGS_ALL immediately   */
 /*             below.                                                */
 /*                                                                   */
 /*             When SCRATCHPAD is defined and FINAL CALL is not, use */
 /*             SQLUDF_TRAIL_ARGS with an additional argument appended*/
 /*             which is a pointer to: SQL_SCRATCHPAD                 */
 /*             E.G.:  SQL_SCRATCHPAD scratch_ptr                     */
 /*                                                                   */
 /*             When DBINFO is defined, use an additional argument    */
 /*             appended at the end of the above argument list.       */
 /*             This argument is a pointer to: SQLUDF_DBINFO          */
 /*********************************************************************/

 %dcl SQLUDF_TRAIL_ARGS char;
 %SQLUDF_TRAIL_ARGS
    = 'sqludf_sqlstate, sqludf_fname, sqludf_fspecname, sqludf_msgtext';
 %dcl SQLUDF_TRAIL_ARGS_DCL char;
 %SQLUDF_TRAIL_ARGS_DCL
    = 'dcl sqludf_sqlstate  char (SQLUDF_SQLSTATE_LEN) varz byaddr, ' ||
          'sqludf_fname     char (SQLUDF_FQNAME_LEN)   varz byaddr, ' ||
          'sqludf_fspecname char (SQLUDF_SPECNAME_LEN) varz byaddr, ' ||
          'sqludf_msgtext   char (SQLUDF_MSGTEXT_LEN)  varz byaddr; ';

 /*********************************************************************/
 /* For the case WITH scratchpad / final_call arguments:              */
 /* SQLUDF_TRAIL_ARGS_ALL                                             */
 /* defines the trailing arguments which are present-- pointers to:   */
 /*               sqlstate,                                           */
 /*               fully qualified function name,                      */
 /*               function specific name,                             */
 /*               message text,                                       */
 /*               scratchpad,                                         */
 /*               call type                                           */
 /*                                                                   */
 /* Usage note: this can be used when                                 */
 /*             - SCRATCHPAD and FINAL CALL are both defined for UDF. */
 /*                                                                   */
 /*             But CANNOT be used for the case where SCRATCHPAD is   */
 /*             not defined and FINAL CALL is defined.  In that case  */
 /*             the call type argument immediately follows the        */
 /*             message text argument.                                */
 /*********************************************************************/

 %dcl SQLUDF_TRAIL_ARGS_ALL char;
 %SQLUDF_TRAIL_ARGS_ALL
   = 'sqludf_sqlstate, sqludf_fname, sqludf_fspecname, '    ||
     'sqludf_msgtext, sqludf_scratchpad, sqludf_call_type';
 %dcl SQLUDF_TRAIL_ARGS_ALL_DCL char;
 %SQLUDF_TRAIL_ARGS_ALL_DCL
    = 'dcl sqludf_sqlstate  char (SQLUDF_SQLSTATE_LEN) varz byaddr, ' ||
          'sqludf_fname     char (SQLUDF_FQNAME_LEN)   varz byaddr, ' ||
          'sqludf_fspecname char (SQLUDF_SPECNAME_LEN) varz byaddr, ' ||
          'sqludf_msgtext   char (SQLUDF_MSGTEXT_LEN)  varz byaddr, ' ||
          'sqludf_scratchpad type SQLUDF_SCRATCHPAD byaddr, '         ||
          'sqludf_call_type type SQLUDF_CALL_TYPE byaddr;';

 /* Macros for referencing the trailing input arguments */
 /* are not needed in PL/I */

 /*********************************************************************/
 /* Special sqlstate (SQLUDF_STATE) values that UDF can set.          */
 /* Usage note: the values "38xxx" and "01Hxx" are included           */
 /*             for completeness and for their educational            */
 /*             value.  You will want to define and use your          */
 /*             won constants with your own values                    */
 /*             substituted for the generic "x"s.                     */
 /*********************************************************************/
 dcl SQLUDF_STATE_OK            char value("00000");
                                /* good return, no problem -         */
                                /* SQLUDF_STATE is init'd to this    */
                                /* value by DB2 before calling UDF.  */
 dcl SQLUDF_STATE_SQLCALL       char value("38000");
                                /* error- UDF issued SQL call,       */
                                /* DB2 returned error (-487) to UDF. */
 dcl SQLUDF_STATE_ERR           char value("38xxx");
                                /* error- xxx is UDF-defined, UDF    */
                                /* should overwrite, but NOT USE     */
                                /* values 0xx thru 5xx, as these are */
                                /* reserved by IBM.                  */
 dcl SQLUDF_STATE_WARN          char value("01Hxx");
                                /* warning - xx is UDF-defined, UDF  */
                                /* should overwrite, any values OK.  */


 /**********************************************/
 /* Macros, for testing for argument nullness: */
 /**********************************************/

 SQLUDF_NULL: proc (nullind) returns (bit(1))
              options (inline) internal;
   dcl nullind type SQLUDF_NULLIND byaddr;
   return (nullind < 0);
 end SQLUDF_NULL;


 SQLUDF_NOTNULL: proc (nullind) returns (bit(1))
                 options (inline) internal;
   dcl nullind type SQLUDF_NULLIND byaddr;
   return (nullind >= 0);
 end SQLUDF_NOTNULL;


 /*********************************************************************/
 /* Return Codes from LOB Locator APIs...                             */
 /* (sqludf_length,  sqludf_substr,                                   */
 /*  sqludf_append,  sqludf_create_locator,                           */
 /*  sqludf_free_locator )                                            */
 /*********************************************************************/

 dcl SQLUDF_API_RC_OK           fixed bin(31) value(0);
                                /* Successful execution.              */
 dcl SQLUDF_API_RC_NOT_UNFENCED fixed bin(31) value(-2);
                                /* Attempt to use API from FENCED UDF */
                                /* - must be from NOT FENCED UDF.     */
 dcl SQLUDF_API_RC_BAD_INPUT    fixed bin(31) value(-3);
                                /* Bad input passed to the API - bad  */
                                /* pointer or length value.           */
 dcl SQLUDF_API_RC_INPUT_LOC    fixed bin(31) value(-4);
                                /* Attempt to free input locator -    */
                                /* UDF not permitted to free a        */
                                /* locator used as input to it.       */

 /*********************************************************************/
 /* API definitions, for LOB Locator APIs                             */
 /* used from within UDFs to manipulate LOB                           */
 /* locators and the data they represent.                             */
 /*********************************************************************/

 /*********************************************************************/
 /* sqludf_length() - API to fetch the length of the LOB              */
 /* value represented by a LOB locator.                               */
 /*********************************************************************/
 dcl sqludf_length
     entry (type udf_locator byaddr,/* in:  Pointer to user-provided  */
                                    /*      LOB locator value.        */
            fixed bin(31) byaddr)   /* in/out: Pointer to where LOB   */
                                    /*      length is to be returned. */
     returns (byvalue fixed bin(31) optional)
     SQL_API_FN external('sqludf_length');


 /*********************************************************************/
 /* sqludf_substr() - API to fetch n contiguous bytes from            */
 /* some byte position within a LOB value represented by              */
 /* a LOB locator.                                                    */
 /*********************************************************************/
 dcl sqludf_substr
     entry (type udf_locator byaddr,/* in:  Pointer to user-provided  */
                                    /*       LOB locator value.       */
            fixed bin(31),          /* in:  Substring start value,    */
                                    /*      first byte is byte 1.     */
            fixed bin(31),          /* in:  Number of bytes to be     */
                                    /*      returned.                 */
            pointer,                /* in/out: Pointer to the buffer  */
                                    /*      into which the bytes are  */
                                    /*      to be placed.             */
            fixed bin(31) byaddr)   /* in/out: Pointer to where the   */
                                    /*      number of bytes actually  */
                                    /*      returned is to be placed  */
                                    /*      (can be smaller than      */
                                    /*       length).                 */
     returns (byvalue fixed bin(31) optional)
     SQL_API_FN external('sqludf_substr');


 /*********************************************************************/
 /* sqludf_append() - API to append n bytes to the end of             */
 /* the LOB value represented by a LOB locator.                       */
 /*********************************************************************/
 dcl sqludf_append
     entry (type udf_locator byaddr,/* in:  Pointer to user-provided  */
                                    /*      LOB locator value.        */
            pointer,                /* in:  Pointer to the buffer con-*/
                                    /*      taining the bytes which   */
                                    /*      are to be appended.       */
            fixed bin(31),          /* in:  Number of bytes to be     */
                                    /*      appended.                 */
            fixed bin(31) byaddr)   /* in/out: Pointer to where the   */
                                    /*      number of bytes actually  */
                                    /*      appended is to be placed. */
     returns (byvalue fixed bin(31) optional)
     SQL_API_FN external('sqludf_append');


 /*********************************************************************/
 /* sqludf_create_locator() - API to create a LOB locator,            */
 /* either to return to DB2 from the UDF or for internal              */
 /* purposes.                                                         */
 /*********************************************************************/
 dcl sqludf_create_locator
     entry (fixed bin (31),         /* in:  Type of LOB, can be any   */
                                    /*      of the types from sql.cpy */
                                    /*      representing LOBs,        */
                                    /*      for example:              */
                                    /*      SQL_TYP_CLOB   for CLOB,  */
                                    /*      SQL_TYP_BLOB   for BLOB,  */
                                    /*      SQL_TYP_DBCLOB for DBCLOB.*/
            pointer byaddr)         /* in/out:  Pointer to a preset   */
                                    /*      pointer to where the      */
                                    /*      locator is to be returned.*/
     returns (byvalue fixed bin(31) optional)
     SQL_API_FN external('sqludf_create_locator');


 /*********************************************************************/
 /* sqludf_free_locator() - API to free a LOB locator                 */
 /* previously created on this call to the UDF;                       */
 /* LOB locators returned to DB2 should not be freed by               */
 /* the UDF.                                                          */
 /*********************************************************************/
 dcl sqludf_free_locator
     entry (type udf_locator byaddr)/* in:  Pointer to user-provided  */
                                    /*      LOB locator value.        */
     returns (byvalue fixed bin(31) optional)
     SQL_API_FN external('sqludf_free_locator');

 %end; /* SQL_H_SQLUDF */
