 /*BEGIN CMC INTERFACE */

 %dcl #XCMC_H char ext;
 %if #XCMC_H = '' %then %do;
 %#XCMC_H = 'Y';

 /*********************************************************************/
 /* PL/I Note: There is no import LIB for the Common Messaging Calls  */
 /* (CMC) applications programming interface (API), as CMC is an      */
 /* optional component of Windows (it need not be installed).         */
 /* The application programmer is expected to use LoadLibrary and     */
 /* GetProcAddress. See the Microsoft Platform SDK for details.       */
 /* In PL/I, you can also use the FETCH statement, which is easier.   */
 /*                                                                   */
 /* The MS Platform SDK says to ensure CMC is installed by looking up */
 /* the value of CMC variable in the registry under                   */
 /* HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows Messaging Subsystem */
 /* and checking that it is 1. If so, extract the value of the        */
 /* CMCDLLNAME32 variable (using CMC.DLL if the variable does not     */
 /* exist or is empty), load the referenced library and obtain the    */
 /* procedure addresses (you can use FETCH and trap ONCODE() = 9250). */
 /*                                                                   */
 /* Microsoft Windows Messaging implements CMC in MAPI32.DLL. So to   */
 /* make things easier, the entry declarations indicate that the CMC  */
 /* routines be loaded from MAPI32 automatically on call.             */
 /* If the CMCDLLNAME32 variable yields anything but MAPI32.DLL, you  */
 /* will need to use the PL/I FETCH statement to load the entries.    */
 /*********************************************************************/

 %dcl CMCAPI_API char ext;
 %CMCAPI_API
   = 'options (byvalue linkage(stdcall) nodescriptor fetchable)';

 /*BASIC DATA TYPES*/
 %dcl DIFFERENT_PLATFORM char ext;
 %if DIFFERENT_PLATFORM = '' %then %do;
 define alias CMC_sint8  fixed bin(7);
 define alias CMC_sint16 fixed bin(15);
 define alias CMC_sint32 fixed bin(31);
 define alias CMC_uint16 unsigned fixed bin(16);
 define alias CMC_uint32 unsigned fixed bin(32);
 define alias CMC_buffer pointer;
 define alias CMC_string pointer;
 %end;

 define alias CMC_boolean           type CMC_uint16;
 define alias CMC_enum              type CMC_sint32;
 define alias CMC_return_code       type CMC_uint32;
 define alias CMC_flags             type CMC_uint32;
 define alias CMC_object_identifier type CMC_string;

 dcl CMC_FALSE type CMC_boolean value(0);
 dcl CMC_TRUE  type CMC_boolean value(1);

 /*DATA STRUCTURES*/

 /*COUNTED STRING*/
 define structure
   1 CMC_counted_string,
     2 length type CMC_uint32,
     2 string  char(1);        /* actually refers, not varz */


 /*SESSION ID*/
 define alias CMC_session_id type CMC_uint32;

 /*TIME*/
 /* unusedX fields needed to align struct on 4-byte boundary */
 define structure
   1 CMC_time,
     2 second  type CMC_sint8,
     2 minute  type CMC_sint8,
     2 hour    type CMC_sint8,
     2 day     type CMC_sint8,
     2 month   type CMC_sint8,
     2 year    type CMC_sint8, /* GRRR! limits to 2027 */
     2 isdst   type CMC_sint8,
     2 unused1 type CMC_sint8,
     2 tmzone  type CMC_sint16,
     2 unused2 type CMC_sint16;

 dcl CMC_NO_TIMEZONE type CMC_sint16  value(-32768 /* '8000'xn*/ );

 define alias CMC_ui_id type CMC_uint32;

 /*EXTENSION*/
 define structure
   1 CMC_extension,
     2 item_code       type CMC_uint32,
     2 item_data       type CMC_uint32,
     2 item_reference  type CMC_buffer,
     2 extension_flags type CMC_flags;

 /* EXTENSION FLAGS */
 dcl CMC_EXT_REQUIRED       type CMC_flags value('00010000'xu);
 dcl CMC_EXT_OUTPUT         type CMC_flags value('00020000'xu);
 dcl CMC_EXT_LAST_ELEMENT   type CMC_flags value('80000000'xu);
 dcl CMC_EXT_RSV_FLAG_MASK  type CMC_flags value('FFFF0000'xu);
 dcl CMC_EXT_ITEM_FLAG_MASK type CMC_flags value('0000FFFF'xu);

 /*ATTACHMENT*/
 define structure
   1 CMC_attachment,
     2 attach_title      type CMC_string,
     2 attach_type       type CMC_object_identifier,
     2 attach_filename   type CMC_string,
     2 attach_flags      type CMC_flags,
     2 attach_extensions handle CMC_extension;

 /* ATTACHMENT FLAGS */
 dcl CMC_ATT_APP_OWNS_FILE type CMC_flags value(1);
 dcl CMC_ATT_LAST_ELEMENT  type CMC_flags value('80000000'xu);

 dcl CMC_ATT_OID_BINARY    char value('? ? ? ? ? ?');
 dcl CMC_ATT_OID_TEXT      char value('? ? ? ? ? ?');

 /*MESSAGE REFERENCE*/
 define alias CMC_message_reference type CMC_counted_string;

 /*RECIPIENT*/
 define structure
   1 CMC_recipient,
     2 name             type CMC_string,
     2 name_type        type CMC_enum,
     2 address          type CMC_string,
     2 role             type CMC_enum,
     2 recip_flags      type CMC_flags,
     2 recip_extensions handle CMC_extension;

 /* NAME TYPES */
 dcl CMC_TYPE_UNKNOWN          type CMC_enum value(0);
 dcl CMC_TYPE_INDIVIDUAL       type CMC_enum value(1);
 dcl CMC_TYPE_GROUP            type CMC_enum value(2);

 /* ROLES */
 dcl CMC_ROLE_TO               type CMC_enum value(0);
 dcl CMC_ROLE_CC               type CMC_enum value(1);
 dcl CMC_ROLE_BCC              type CMC_enum value(2);
 dcl CMC_ROLE_ORIGINATOR       type CMC_enum value(3);
 dcl CMC_ROLE_AUTHORIZING_USER type CMC_enum value(4);

 /* RECIPIENT FLAGS */
 dcl CMC_RECIP_IGNORE          type CMC_flags value(1);
 dcl CMC_RECIP_LIST_TRUNCATED  type CMC_flags value(2);
 dcl CMC_RECIP_LAST_ELEMENT    type CMC_flags value('80000000'xu);

 /*MESSAGE*/
 define structure
   1 CMC_message,
     2 message_reference  handle CMC_message_reference,
     2 message_type       type CMC_string,
     2 subject            type CMC_string,
     2 time_sent          type CMC_time,
     2 text_note          type CMC_string,
     2 recipients         handle CMC_recipient,
     2 attachments        handle CMC_attachment,
     2 message_flags      type CMC_flags,
     2 message_extensions handle CMC_extension;

 /* MESSAGE FLAGS */
 dcl CMC_MSG_READ              type CMC_flags value(1);
 dcl CMC_MSG_TEXT_NOTE_AS_FILE type CMC_flags value(2);
 dcl CMC_MSG_UNSENT            type CMC_flags value(4);
 dcl CMC_MSG_LAST_ELEMENT      type CMC_flags value('80000000'xu);

 /*MESSAGE SUMMARY*/
 define structure
   1 CMC_message_summary,
     2 message_reference          handle CMC_message_reference,
     2 message_type               type CMC_string,
     2 subject                    type CMC_string,
     2 time_sent                  type CMC_time,
     2 byte_length                type CMC_uint32,
     2 originator                 handle CMC_recipient,
     2 summary_flags              type CMC_flags,
     2 message_summary_extensions handle CMC_extension;

 /* MESSAGE SUMMARY FLAGS */
 dcl CMC_SUM_READ         type CMC_flags value(1);
 dcl CMC_SUM_UNSENT       type CMC_flags value(2);
 dcl CMC_SUM_LAST_ELEMENT type CMC_flags value('80000000'xu);

 /*CMC FUNCTIONS */

 /*CROSS FUNCTION FLAGS */
 dcl CMC_ERROR_UI_ALLOWED    type CMC_flags value('01000000'xu);
 dcl CMC_LOGON_UI_ALLOWED    type CMC_flags value('02000000'xu);
 dcl CMC_COUNTED_STRING_TYPE type CMC_flags value('04000000'xu);

 /*SEND*/
 dcl cmc_send
     entry (type CMC_session_id,
            type CMC_message byaddr,
            type CMC_flags,
            type CMC_ui_id,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_send');

 dcl CMC_SEND_UI_REQUESTED type CMC_flags value(1);

 /*SEND DOCUMENT*/
 dcl cmc_send_documents
     entry (char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            type CMC_flags,
            char (*) varz byaddr,
            char (*) varz byaddr optional,
            char (1) byaddr,
            type CMC_ui_id)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_send_documents');

 dcl CMC_FIRST_ATTACH_AS_TEXT_NOTE type CMC_flags value(2);

 /*ACT ON*/
 dcl cmc_act_on
     entry (type CMC_session_id,
            type CMC_message_reference byaddr,
            type CMC_enum,
            type CMC_flags,
            type CMC_ui_id,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_act_on');

 dcl CMC_ACT_ON_EXTENDED type CMC_enum value(0);
 dcl CMC_ACT_ON_DELETE   type CMC_enum value(1);

 /*LIST*/
 dcl cmc_list
     entry (type CMC_session_id,
            char (*) varz byaddr optional,
            type CMC_flags,
            type CMC_message_reference byaddr optional,
            type CMC_uint32 byaddr,
            type CMC_ui_id,
            handle CMC_message_summary byaddr,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_list');

 dcl CMC_LIST_UNREAD_ONLY   type CMC_flags value(1);
 dcl CMC_LIST_MSG_REFS_ONLY type CMC_flags value(2);
 dcl CMC_LIST_COUNT_ONLY    type CMC_flags value(4);

 dcl CMC_LENGTH_UNKNOWN     fixed bin(31) value('FFFFFFFF'xn);

 /*READ*/
 dcl cmc_read
     entry (type CMC_session_id,
            type CMC_message_reference byaddr,
            type CMC_flags,
            handle CMC_message byaddr,
            type CMC_ui_id,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_read');

 dcl CMC_DO_NOT_MARK_AS_READ       type CMC_flags value(1);
 dcl CMC_MSG_AND_ATT_HDRS_ONLY     type CMC_flags value(2);
 dcl CMC_READ_FIRST_UNREAD_MESSAGE type CMC_flags value(4);

 /*LOOK UP*/
 dcl cmc_look_up
     entry (type CMC_session_id,
            (*) type CMC_recipient byaddr,
            type CMC_flags,
            type CMC_ui_id,
            type CMC_uint32 byaddr,
            handle CMC_recipient byaddr,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_look_up');

 dcl CMC_LOOKUP_RESOLVE_PREFIX_SEARCH type CMC_flags value(1);
 dcl CMC_LOOKUP_RESOLVE_IDENTITY      type CMC_flags value(2);
 dcl CMC_LOOKUP_RESOLVE_UI            type CMC_flags value(4);
 dcl CMC_LOOKUP_DETAILS_UI            type CMC_flags value(8);
 dcl CMC_LOOKUP_ADDRESSING_UI         type CMC_flags value(16);

 /*FREE*/
 dcl cmc_free
     entry (type CMC_buffer)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_free');

 /* LOGOFF */
 dcl cmc_logoff
     entry (type CMC_session_id,
            type CMC_ui_id,
            type CMC_flags,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_logoff');

 dcl CMC_LOGOFF_UI_ALLOWED type CMC_flags value(1);

 /* LOGON */
 dcl cmc_logon
     entry (char(*) varz byaddr optional,
            char(*) varz byaddr optional,
            char(*) varz byaddr optional,
            type CMC_object_identifier,
            type CMC_ui_id,
            type CMC_uint16,
            type CMC_flags,
            type CMC_session_id byaddr,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_logon');

 dcl CMC_VERSION type CMC_uint16 value(100);

 /* QUERY CONFIGURATION */
 dcl cmc_query_configuration
     entry (type CMC_session_id,
            type CMC_enum,
            type CMC_buffer,
            (*) type CMC_extension byaddr optional)
     returns (type CMC_return_code byvalue optional)
     CMCAPI_API external('MAPI32/cmc_query_configuration');

 /*QUERY CONFIGURATION ENUMS */
 dcl CMC_CONFIG_CHARACTER_SET   type CMC_enum value(1);
 dcl CMC_CONFIG_LINE_TERM       type CMC_enum value(2);
 dcl CMC_CONFIG_DEFAULT_SERVICE type CMC_enum value(3);
 dcl CMC_CONFIG_DEFAULT_USER    type CMC_enum value(4);
 dcl CMC_CONFIG_REQ_PASSWORD    type CMC_enum value(5);
 dcl CMC_CONFIG_REQ_SERVICE     type CMC_enum value(6);
 dcl CMC_CONFIG_REQ_USER        type CMC_enum value(7);
 dcl CMC_CONFIG_UI_AVAIL        type CMC_enum value(8);
 dcl CMC_CONFIG_SUP_NOMKMSGREAD type CMC_enum value(9);
 dcl CMC_CONFIG_SUP_COUNTED_STR type CMC_enum value(10);
 dcl CMC_CONFIG_VER_IMPLEM      type CMC_enum value(11);
 dcl CMC_CONFIG_VER_SPEC        type CMC_enum value(12);

 /* CONFIG LINE TERM ENUM */
 dcl CMC_LINE_TERM_CRLF         type CMC_enum value(0);
 dcl CMC_LINE_TERM_CR           type CMC_enum value(1);
 dcl CMC_LINE_TERM_LF           type CMC_enum value(2);

 /* CONFIG REQUIRED LOGON PARAMETER ENUM */
 dcl CMC_REQUIRED_NO            type CMC_enum value(0);
 dcl CMC_REQUIRED_YES           type CMC_enum value(1);
 dcl CMC_REQUIRED_OPT           type CMC_enum value(2);

 /* DEFINED OBJECT ID'S FOR CHARACTER SETS */
 dcl CMC_CHAR_CP437             char value('1 2 840 113556 3 2 437');
 dcl CMC_CHAR_CP850             char value('1 2 840 113556 3 2 850');
 dcl CMC_CHAR_CP1252            char value('1 2 840 113556 3 2 1252');
 dcl CMC_CHAR_ISTRING           char value('1 2 840 113556 3 2 0');
 dcl CMC_CHAR_UNICODE           char value('1 2 840 113556 3 2 1');

 /* RETURN CODE FLAGS */
 dcl CMC_ERROR_DISPLAYED       type CMC_return_code value('00008000'xu);
 dcl CMC_ERROR_RSV_MASK        type CMC_return_code value('0000FFFF'xu);
 dcl CMC_ERROR_IMPL_MASK       type CMC_return_code value('FFFF0000'xu);

 /* RETURN CODES */
 dcl CMC_SUCCESS                      type CMC_return_code value(0);

 dcl CMC_E_AMBIGUOUS_RECIPIENT        type CMC_return_code value(1);
 dcl CMC_E_ATTACHMENT_NOT_FOUND       type CMC_return_code value(2);
 dcl CMC_E_ATTACHMENT_OPEN_FAILURE    type CMC_return_code value(3);
 dcl CMC_E_ATTACHMENT_READ_FAILURE    type CMC_return_code value(4);
 dcl CMC_E_ATTACHMENT_WRITE_FAILURE   type CMC_return_code value(5);
 dcl CMC_E_COUNTED_STRING_UNSUPPORTED type CMC_return_code value(6);
 dcl CMC_E_DISK_FULL                  type CMC_return_code value(7);
 dcl CMC_E_FAILURE                    type CMC_return_code value(8);
 dcl CMC_E_INSUFFICIENT_MEMORY        type CMC_return_code value(9);
 dcl CMC_E_INVALID_CONFIGURATION      type CMC_return_code value(10);
 dcl CMC_E_INVALID_ENUM               type CMC_return_code value(11);
 dcl CMC_E_INVALID_FLAG               type CMC_return_code value(12);
 dcl CMC_E_INVALID_MEMORY             type CMC_return_code value(13);
 dcl CMC_E_INVALID_MESSAGE_PARAMETER  type CMC_return_code value(14);
 dcl CMC_E_INVALID_MESSAGE_REFERENCE  type CMC_return_code value(15);
 dcl CMC_E_INVALID_PARAMETER          type CMC_return_code value(16);
 dcl CMC_E_INVALID_SESSION_ID         type CMC_return_code value(17);
 dcl CMC_E_INVALID_UI_ID              type CMC_return_code value(18);
 dcl CMC_E_LOGON_FAILURE              type CMC_return_code value(19);
 dcl CMC_E_MESSAGE_IN_USE             type CMC_return_code value(20);
 dcl CMC_E_NOT_SUPPORTED              type CMC_return_code value(21);
 dcl CMC_E_PASSWORD_REQUIRED          type CMC_return_code value(22);
 dcl CMC_E_RECIPIENT_NOT_FOUND        type CMC_return_code value(23);
 dcl CMC_E_SERVICE_UNAVAILABLE        type CMC_return_code value(24);
 dcl CMC_E_TEXT_TOO_LARGE             type CMC_return_code value(25);
 dcl CMC_E_TOO_MANY_FILES             type CMC_return_code value(26);
 dcl CMC_E_TOO_MANY_RECIPIENTS        type CMC_return_code value(27);
 dcl CMC_E_UNABLE_TO_NOT_MARK_AS_READ type CMC_return_code value(28);
 dcl CMC_E_UNRECOGNIZED_MESSAGE_TYPE  type CMC_return_code value(29);
 dcl CMC_E_UNSUPPORTED_ACTION         type CMC_return_code value(30);
 dcl CMC_E_UNSUPPORTED_CHARACTER_SET  type CMC_return_code value(31);
 dcl CMC_E_UNSUPPORTED_DATA_EXT       type CMC_return_code value(32);
 dcl CMC_E_UNSUPPORTED_FLAG           type CMC_return_code value(33);
 dcl CMC_E_UNSUPPORTED_FUNCTION_EXT   type CMC_return_code value(34);
 dcl CMC_E_UNSUPPORTED_VERSION        type CMC_return_code value(35);
 dcl CMC_E_USER_CANCEL                type CMC_return_code value(36);
 dcl CMC_E_USER_NOT_LOGGED_ON         type CMC_return_code value(37);

 %end; /* _XCMC_H */

